"""
Support for IP Cameras.

This component provides basic support for IP cameras. For the basic support to
work you camera must support accessing a JPEG snapshot via a URL and you will
need to specify the "still_image_url" parameter which should be the location of
the JPEG image.

As part of the basic support the following features will be provided:
-MJPEG video streaming
-Saving a snapshot
-Recording(JPEG frame capture)

To use this component, add the following to your config/configuration.yaml:

camera:
    platform: generic
    name: Door Camera
    username: YOUR_USERNAME
    password: YOUR_PASSWORD
    still_image_url: http://YOUR_CAMERA_IP_AND_PORT/image.jpg


VARIABLES:

These are the variables for the device_data array:

still_image_url
*Required
The URL your camera serves the image on.
Example: http://192.168.1.21:2112/

name
*Optional
This parameter allows you to override the name of your camera in homeassistant

username
*Optional
THe username for acessing your camera

password
*Optional
the password for accessing your camera


"""
import logging
from requests.auth import HTTPBasicAuth
from homeassistant.helpers import validate_config
from homeassistant.components.camera import DOMAIN
from homeassistant.components.camera import Camera
import requests

_LOGGER = logging.getLogger(__name__)


# pylint: disable=unused-argument
def setup_platform(hass, config, add_devices_callback, discovery_info=None):
    """ Adds a generic IP Camera. """
    if not validate_config({DOMAIN: config}, {DOMAIN: ['still_image_url']},
                           _LOGGER):
        return None

    add_devices_callback([GenericCamera(config)])


# pylint: disable=too-many-instance-attributes
class GenericCamera(Camera):
    """
    A generic implementation of an IP camera that is reachable over a URL.
    """

    def __init__(self, device_info):
        super().__init__()
        self._name = device_info.get('name', 'Generic Camera')
        self._username = device_info.get('username')
        self._password = device_info.get('password')
        self._still_image_url = device_info['still_image_url']

    def camera_image(self):
        """ Return a still image reponse from the camera """
        if self._username and self._password:
            response = requests.get(
                self._still_image_url,
                auth=HTTPBasicAuth(self._username, self._password))
        else:
            response = requests.get(self._still_image_url)

        return response.content

    @property
    def name(self):
        """ Return the name of this device """
        return self._name



"""
homeassistant.components.device_tracker.ddwrt
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Device tracker platform that supports scanning a DD-WRT router for device
presence.

Configuration:

To use the DD-WRT tracker you will need to add something like the following
to your config/configuration.yaml

device_tracker:
  platform: ddwrt
  host: YOUR_ROUTER_IP
  username: YOUR_ADMIN_USERNAME
  password: YOUR_ADMIN_PASSWORD

Variables:

host
*Required
The IP address of your router, e.g. 192.168.1.1.

username
*Required
The username of an user with administrative privileges, usually 'admin'.

password
*Required
The password for your given admin account.
"""
import logging
from datetime import timedelta
import re
import threading
import requests

from homeassistant.const import CONF_HOST, CONF_USERNAME, CONF_PASSWORD
from homeassistant.helpers import validate_config
from homeassistant.util import Throttle
from homeassistant.components.device_tracker import DOMAIN

# Return cached results if last scan was less then this time ago
MIN_TIME_BETWEEN_SCANS = timedelta(seconds=5)

_LOGGER = logging.getLogger(__name__)

_DDWRT_DATA_REGEX = re.compile(r'\{(\w+)::([^\}]*)\}')


# pylint: disable=unused-argument
def get_scanner(hass, config):
    """ Validates config and returns a DD-WRT scanner. """
    if not validate_config(config,
                           {DOMAIN: [CONF_HOST, CONF_USERNAME, CONF_PASSWORD]},
                           _LOGGER):
        return None

    scanner = DdWrtDeviceScanner(config[DOMAIN])

    return scanner if scanner.success_init else None


# pylint: disable=too-many-instance-attributes
class DdWrtDeviceScanner(object):
    """ This class queries a wireless router running DD-WRT firmware
    for connected devices. Adapted from Tomato scanner.
    """

    def __init__(self, config):
        self.host = config[CONF_HOST]
        self.username = config[CONF_USERNAME]
        self.password = config[CONF_PASSWORD]

        self.lock = threading.Lock()

        self.last_results = {}

        self.mac2name = None

        # Test the router is accessible
        url = 'http://{}/Status_Wireless.live.asp'.format(self.host)
        data = self.get_ddwrt_data(url)
        self.success_init = data is not None

    def scan_devices(self):
        """ Scans for new devices and return a
            list containing found device ids. """

        self._update_info()

        return self.last_results

    def get_device_name(self, device):
        """ Returns the name of the given device or None if we don't know. """

        with self.lock:
            # if not initialised and not already scanned and not found
            if self.mac2name is None or device not in self.mac2name:
                url = 'http://{}/Status_Lan.live.asp'.format(self.host)
                data = self.get_ddwrt_data(url)

                if not data:
                    return

                dhcp_leases = data.get('dhcp_leases', None)
                if dhcp_leases:
                    # remove leading and trailing single quotes
                    cleaned_str = dhcp_leases.strip().strip('"')
                    elements = cleaned_str.split('","')
                    num_clients = int(len(elements)/5)
                    self.mac2name = {}
                    for idx in range(0, num_clients):
                        # this is stupid but the data is a single array
                        # every 5 elements represents one hosts, the MAC
                        # is the third element and the name is the first
                        mac_index = (idx * 5) + 2
                        if mac_index < len(elements):
                            mac = elements[mac_index]
                            self.mac2name[mac] = elements[idx * 5]

            return self.mac2name.get(device, None)

    @Throttle(MIN_TIME_BETWEEN_SCANS)
    def _update_info(self):
        """ Ensures the information from the DD-WRT router is up to date.
            Returns boolean if scanning successful. """
        if not self.success_init:
            return False

        with self.lock:
            _LOGGER.info("Checking ARP")

            url = 'http://{}/Status_Wireless.live.asp'.format(self.host)
            data = self.get_ddwrt_data(url)

            if not data:
                return False

            if data:
                self.last_results = []
                active_clients = data.get('active_wireless', None)
                if active_clients:
                    # This is really lame, instead of using JSON the DD-WRT UI
                    # uses its own data format for some reason and then
                    # regex's out values so I guess I have to do the same,
                    # LAME!!!

                    # remove leading and trailing single quotes
                    clean_str = active_clients.strip().strip("'")
                    elements = clean_str.split("','")

                    num_clients = int(len(elements)/9)
                    for idx in range(0, num_clients):
                        # get every 9th element which is the MAC address
                        index = idx * 9
                        if index < len(elements):
                            self.last_results.append(elements[index])

                    return True

            return False

    def get_ddwrt_data(self, url):
        """ Retrieve data from DD-WRT and return parsed result.  """
        try:
            response = requests.get(
                url,
                auth=(self.username, self.password),
                timeout=4)
        except requests.exceptions.Timeout:
            _LOGGER.exception("Connection to the router timed out")
            return
        if response.status_code == 200:
            return _parse_ddwrt_response(response.text)
        elif response.status_code == 401:
            # Authentication error
            _LOGGER.exception(
                "Failed to authenticate, "
                "please check your username and password")
            return
        else:
            _LOGGER.error("Invalid response from ddwrt: %s", response)


def _parse_ddwrt_response(data_str):
    """ Parse the DD-WRT data format. """
    return {
        key: val for key, val in _DDWRT_DATA_REGEX
        .findall(data_str)}



"""
homeassistant.components.light.limitlessled
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Support for LimitlessLED bulbs, also known as...

- EasyBulb
- AppLight
- AppLamp
- MiLight
- LEDme
- dekolight
- iLight

Configuration:

To use limitlessled you will need to add the following to your
config/configuration.yaml.

light:
  platform: limitlessled
  host: 192.168.1.10
  group_1_name: Living Room
  group_2_name: Bedroom
  group_3_name: Office
  group_4_name: Kitchen

"""
import logging

from homeassistant.const import DEVICE_DEFAULT_NAME
from homeassistant.components.light import (Light, ATTR_BRIGHTNESS,
                                            ATTR_XY_COLOR)
from homeassistant.util.color import color_RGB_to_xy

_LOGGER = logging.getLogger(__name__)
REQUIREMENTS = ['ledcontroller==1.0.7']


def setup_platform(hass, config, add_devices_callback, discovery_info=None):
    """ Gets the LimitlessLED lights. """
    import ledcontroller

    led = ledcontroller.LedController(config['host'])

    lights = []
    for i in range(1, 5):
        if 'group_%d_name' % (i) in config:
            lights.append(LimitlessLED(led, i, config['group_%d_name' % (i)]))

    add_devices_callback(lights)


class LimitlessLED(Light):
    """ Represents a LimitlessLED light """

    def __init__(self, led, group, name):
        self.led = led
        self.group = group

        # LimitlessLEDs don't report state, we have track it ourselves.
        self.led.off(self.group)

        self._name = name or DEVICE_DEFAULT_NAME
        self._state = False
        self._brightness = 100
        self._xy_color = color_RGB_to_xy(255, 255, 255)

        # Build a color table that maps an RGB color to a color string
        # recognized by LedController's set_color method
        self._color_table = [(color_RGB_to_xy(*x[0]), x[1]) for x in [
            ((0xFF, 0xFF, 0xFF), 'white'),
            ((0xEE, 0x82, 0xEE), 'violet'),
            ((0x41, 0x69, 0xE1), 'royal_blue'),
            ((0x87, 0xCE, 0xFA), 'baby_blue'),
            ((0x00, 0xFF, 0xFF), 'aqua'),
            ((0x7F, 0xFF, 0xD4), 'royal_mint'),
            ((0x2E, 0x8B, 0x57), 'seafoam_green'),
            ((0x00, 0x80, 0x00), 'green'),
            ((0x32, 0xCD, 0x32), 'lime_green'),
            ((0xFF, 0xFF, 0x00), 'yellow'),
            ((0xDA, 0xA5, 0x20), 'yellow_orange'),
            ((0xFF, 0xA5, 0x00), 'orange'),
            ((0xFF, 0x00, 0x00), 'red'),
            ((0xFF, 0xC0, 0xCB), 'pink'),
            ((0xFF, 0x00, 0xFF), 'fusia'),
            ((0xDA, 0x70, 0xD6), 'lilac'),
            ((0xE6, 0xE6, 0xFA), 'lavendar'),
        ]]

    @property
    def should_poll(self):
        """ No polling needed for a demo light. """
        return False

    @property
    def name(self):
        """ Returns the name of the device if any. """
        return self._name

    @property
    def brightness(self):
        return self._brightness

    @property
    def color_xy(self):
        return self._xy_color

    def _xy_to_led_color(self, xy_color):
        """ Convert an XY color to the closest LedController color string """
        def abs_dist_squared(p_0, p_1):
            """ Returns the absolute value of the squared distance """
            return abs((p_0[0] - p_1[0])**2 + (p_0[1] - p_1[1])**2)

        candidates = [(abs_dist_squared(xy_color, x[0]), x[1]) for x in
                      self._color_table]

        # First candidate in the sorted list is closest to desired color:
        return sorted(candidates)[0][1]

    @property
    def is_on(self):
        """ True if device is on. """
        return self._state

    def turn_on(self, **kwargs):
        """ Turn the device on. """
        self._state = True

        if ATTR_BRIGHTNESS in kwargs:
            self._brightness = kwargs[ATTR_BRIGHTNESS]

        if ATTR_XY_COLOR in kwargs:
            self._xy_color = kwargs[ATTR_XY_COLOR]

        self.led.set_color(self._xy_to_led_color(self._xy_color), self.group)
        self.led.set_brightness(self._brightness / 255.0, self.group)
        self.update_ha_state()

    def turn_off(self, **kwargs):
        """ Turn the device off. """
        self._state = False
        self.led.off(self.group)
        self.update_ha_state()



"""
homeassistant.components.device_tracker.tplink
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Device tracker platform that supports scanning a TP-Link router for device
presence.

Configuration:

To use the TP-Link tracker you will need to add something like the following
to your config/configuration.yaml

device_tracker:
  platform: tplink
  host: YOUR_ROUTER_IP
  username: YOUR_ADMIN_USERNAME
  password: YOUR_ADMIN_PASSWORD

Variables:

host
*Required
The IP address of your router, e.g. 192.168.1.1.

username
*Required
The username of an user with administrative privileges, usually 'admin'.

password
*Required
The password for your given admin account.

"""
import base64
import logging
from datetime import timedelta
import re
import threading
import requests

from homeassistant.const import CONF_HOST, CONF_USERNAME, CONF_PASSWORD
from homeassistant.helpers import validate_config
from homeassistant.util import Throttle
from homeassistant.components.device_tracker import DOMAIN

# Return cached results if last scan was less then this time ago
MIN_TIME_BETWEEN_SCANS = timedelta(seconds=5)

_LOGGER = logging.getLogger(__name__)


def get_scanner(hass, config):
    """ Validates config and returns a TP-Link scanner. """
    if not validate_config(config,
                           {DOMAIN: [CONF_HOST, CONF_USERNAME, CONF_PASSWORD]},
                           _LOGGER):
        return None

    scanner = Tplink2DeviceScanner(config[DOMAIN])

    if not scanner.success_init:
        scanner = TplinkDeviceScanner(config[DOMAIN])

    return scanner if scanner.success_init else None


class TplinkDeviceScanner(object):
    """ This class queries a wireless router running TP-Link firmware
    for connected devices.
    """

    def __init__(self, config):
        host = config[CONF_HOST]
        username, password = config[CONF_USERNAME], config[CONF_PASSWORD]

        self.parse_macs = re.compile('[0-9A-F]{2}-[0-9A-F]{2}-[0-9A-F]{2}-' +
                                     '[0-9A-F]{2}-[0-9A-F]{2}-[0-9A-F]{2}')

        self.host = host
        self.username = username
        self.password = password

        self.last_results = {}
        self.lock = threading.Lock()
        self.success_init = self._update_info()

    def scan_devices(self):
        """ Scans for new devices and return a
            list containing found device ids. """

        self._update_info()

        return self.last_results

    # pylint: disable=no-self-use
    def get_device_name(self, device):
        """ The TP-Link firmware doesn't save the name of the wireless
            device. """

        return None

    @Throttle(MIN_TIME_BETWEEN_SCANS)
    def _update_info(self):
        """ Ensures the information from the TP-Link router is up to date.
            Returns boolean if scanning successful. """

        with self.lock:
            _LOGGER.info("Loading wireless clients...")

            url = 'http://{}/userRpm/WlanStationRpm.htm'.format(self.host)
            referer = 'http://{}'.format(self.host)
            page = requests.get(url, auth=(self.username, self.password),
                                headers={'referer': referer})

            result = self.parse_macs.findall(page.text)

            if result:
                self.last_results = [mac.replace("-", ":") for mac in result]
                return True

            return False


class Tplink2DeviceScanner(TplinkDeviceScanner):
    """ This class queries a wireless router running newer version of TP-Link
    firmware for connected devices.
    """

    def scan_devices(self):
        """ Scans for new devices and return a
            list containing found device ids. """

        self._update_info()
        return self.last_results.keys()

    # pylint: disable=no-self-use
    def get_device_name(self, device):
        """ The TP-Link firmware doesn't save the name of the wireless
            device. """

        return self.last_results.get(device)

    @Throttle(MIN_TIME_BETWEEN_SCANS)
    def _update_info(self):
        """ Ensures the information from the TP-Link router is up to date.
            Returns boolean if scanning successful. """

        with self.lock:
            _LOGGER.info("Loading wireless clients...")

            url = 'http://{}/data/map_access_wireless_client_grid.json'\
                .format(self.host)
            referer = 'http://{}'.format(self.host)

            # Router uses Authorization cookie instead of header
            # Let's create the cookie
            username_password = '{}:{}'.format(self.username, self.password)
            b64_encoded_username_password = base64.b64encode(
                username_password.encode('ascii')
            ).decode('ascii')
            cookie = 'Authorization=Basic {}'\
                .format(b64_encoded_username_password)

            response = requests.post(url, headers={'referer': referer,
                                                   'cookie': cookie})

            try:
                result = response.json().get('data')
            except ValueError:
                _LOGGER.error("Router didn't respond with JSON. "
                              "Check if credentials are correct.")
                return False

            if result:
                self.last_results = {
                    device['mac_addr'].replace('-', ':'): device['name']
                    for device in result
                }
                return True

            return False



"""
homeassistant.components.switch.demo
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Demo platform that has two fake switches.

"""
from homeassistant.components.switch import SwitchDevice
from homeassistant.const import DEVICE_DEFAULT_NAME


# pylint: disable=unused-argument
def setup_platform(hass, config, add_devices_callback, discovery_info=None):
    """ Find and return demo switches. """
    add_devices_callback([
        DemoSwitch('Decorative Lights', True),
        DemoSwitch('AC', False)
    ])


class DemoSwitch(SwitchDevice):
    """ Provides a demo switch. """
    def __init__(self, name, state):
        self._name = name or DEVICE_DEFAULT_NAME
        self._state = state

    @property
    def should_poll(self):
        """ No polling needed for a demo switch. """
        return False

    @property
    def name(self):
        """ Returns the name of the device if any. """
        return self._name

    @property
    def current_power_mwh(self):
        """ Current power usage in mwh. """
        if self._state:
            return 100

    @property
    def today_power_mw(self):
        """ Today total power usage in mw. """
        return 1500

    @property
    def is_on(self):
        """ True if device is on. """
        return self._state

    def turn_on(self, **kwargs):
        """ Turn the device on. """
        self._state = True
        self.update_ha_state()

    def turn_off(self, **kwargs):
        """ Turn the device off. """
        self._state = False
        self.update_ha_state()



"""
homeassistant.components.switch.vera
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Support for Vera switches.

Configuration:
To use the Vera lights you will need to add something like the following to
your config/configuration.yaml.

switch:
    platform: vera
    vera_controller_url: http://YOUR_VERA_IP:3480/
    device_data:
        12:
            name: My awesome switch
            exclude: true
        13:
            name: Another Switch

Variables:

vera_controller_url
*Required
This is the base URL of your vera controller including the port number if not
running on 80. Example: http://192.168.1.21:3480/

device_data
*Optional
This contains an array additional device info for your Vera devices.  It is not
required and if not specified all lights configured in your Vera controller
will be added with default values.  You should use the id of your vera device
as the key for the device within device_data.

These are the variables for the device_data array:

name
*Optional
This parameter allows you to override the name of your Vera device in the HA
interface, if not specified the value configured for the device in your Vera
will be used.

exclude
*Optional
This parameter allows you to exclude the specified device from homeassistant,
it should be set to "true" if you want this device excluded.
"""
import logging
import time
from requests.exceptions import RequestException
import homeassistant.util.dt as dt_util

from homeassistant.helpers.entity import ToggleEntity
from homeassistant.const import (
    ATTR_BATTERY_LEVEL, ATTR_TRIPPED, ATTR_ARMED, ATTR_LAST_TRIP_TIME)
# pylint: disable=no-name-in-module, import-error
import homeassistant.external.vera.vera as veraApi

_LOGGER = logging.getLogger(__name__)


# pylint: disable=unused-argument
def get_devices(hass, config):
    """ Find and return Vera switches. """

    base_url = config.get('vera_controller_url')
    if not base_url:
        _LOGGER.error(
            "The required parameter 'vera_controller_url'"
            " was not found in config"
        )
        return False

    device_data = config.get('device_data', {})

    vera_controller = veraApi.VeraController(base_url)
    devices = []
    try:
        devices = vera_controller.get_devices([
            'Switch', 'Armable Sensor', 'On/Off Switch'])
    except RequestException:
        # There was a network related error connecting to the vera controller.
        _LOGGER.exception("Error communicating with Vera API")
        return False

    vera_switches = []
    for device in devices:
        extra_data = device_data.get(device.deviceId, {})
        exclude = extra_data.get('exclude', False)

        if exclude is not True:
            vera_switches.append(VeraSwitch(device, extra_data))

    return vera_switches


def setup_platform(hass, config, add_devices, discovery_info=None):
    """ Find and return Vera lights. """
    add_devices(get_devices(hass, config))


class VeraSwitch(ToggleEntity):
    """ Represents a Vera Switch. """

    def __init__(self, vera_device, extra_data=None):
        self.vera_device = vera_device
        self.extra_data = extra_data
        if self.extra_data and self.extra_data.get('name'):
            self._name = self.extra_data.get('name')
        else:
            self._name = self.vera_device.name
        self.is_on_status = False
        # for debouncing status check after command is sent
        self.last_command_send = 0

    @property
    def name(self):
        """ Get the mame of the switch. """
        return self._name

    @property
    def state_attributes(self):
        attr = super().state_attributes

        if self.vera_device.has_battery:
            attr[ATTR_BATTERY_LEVEL] = self.vera_device.battery_level + '%'

        if self.vera_device.is_armable:
            armed = self.vera_device.refresh_value('Armed')
            attr[ATTR_ARMED] = 'True' if armed == '1' else 'False'

        if self.vera_device.is_trippable:
            last_tripped = self.vera_device.refresh_value('LastTrip')
            if last_tripped is not None:
                utc_time = dt_util.utc_from_timestamp(int(last_tripped))
                attr[ATTR_LAST_TRIP_TIME] = dt_util.datetime_to_str(
                    utc_time)
            else:
                attr[ATTR_LAST_TRIP_TIME] = None
            tripped = self.vera_device.refresh_value('Tripped')
            attr[ATTR_TRIPPED] = 'True' if tripped == '1' else 'False'

        attr['Vera Device Id'] = self.vera_device.vera_device_id

        return attr

    def turn_on(self, **kwargs):
        self.last_command_send = time.time()
        self.vera_device.switch_on()
        self.is_on_status = True

    def turn_off(self, **kwargs):
        self.last_command_send = time.time()
        self.vera_device.switch_off()
        self.is_on_status = False

    @property
    def is_on(self):
        """ True if device is on. """
        return self.is_on_status

    def update(self):
        # We need to debounce the status call after turning switch on or off
        # because the vera has some lag in updating the device status
        if (self.last_command_send + 5) < time.time():
            self.is_on_status = self.vera_device.is_switched_on()



"""
homeassistant
~~~~~~~~~~~~~

Home Assistant is a Home Automation framework for observing the state
of entities and react to changes.
"""

import os
import time
import logging
import signal
import threading
import enum
import re
import functools as ft
from collections import namedtuple

from homeassistant.const import (
    EVENT_HOMEASSISTANT_START, EVENT_HOMEASSISTANT_STOP,
    SERVICE_HOMEASSISTANT_STOP, EVENT_TIME_CHANGED, EVENT_STATE_CHANGED,
    EVENT_CALL_SERVICE, ATTR_NOW, ATTR_DOMAIN, ATTR_SERVICE, MATCH_ALL,
    EVENT_SERVICE_EXECUTED, ATTR_SERVICE_CALL_ID, EVENT_SERVICE_REGISTERED,
    TEMP_CELCIUS, TEMP_FAHRENHEIT, ATTR_FRIENDLY_NAME)
from homeassistant.exceptions import (
    HomeAssistantError, InvalidEntityFormatError)
import homeassistant.util as util
import homeassistant.util.dt as date_util
import homeassistant.helpers.temperature as temp_helper
from homeassistant.config import get_default_config_dir

DOMAIN = "homeassistant"

# How often time_changed event should fire
TIMER_INTERVAL = 1  # seconds

# How long we wait for the result of a service call
SERVICE_CALL_LIMIT = 10  # seconds

# Define number of MINIMUM worker threads.
# During bootstrap of HA (see bootstrap._setup_component()) worker threads
# will be added for each component that polls devices.
MIN_WORKER_THREAD = 2

# Pattern for validating entity IDs (format: <domain>.<entity>)
ENTITY_ID_PATTERN = re.compile(r"^(?P<domain>\w+)\.(?P<entity>\w+)$")

_LOGGER = logging.getLogger(__name__)

# Temporary to support deprecated methods
_MockHA = namedtuple("MockHomeAssistant", ['bus'])


class HomeAssistant(object):
    """ Core class to route all communication to right components. """

    def __init__(self):
        self.pool = pool = create_worker_pool()
        self.bus = EventBus(pool)
        self.services = ServiceRegistry(self.bus, pool)
        self.states = StateMachine(self.bus)
        self.config = Config()

    def start(self):
        """ Start home assistant. """
        _LOGGER.info(
            "Starting Home Assistant (%d threads)", self.pool.worker_count)

        create_timer(self)
        self.bus.fire(EVENT_HOMEASSISTANT_START)

    def block_till_stopped(self):
        """ Will register service homeassistant/stop and
            will block until called. """
        request_shutdown = threading.Event()

        def stop_homeassistant(*args):
            """ Stops Home Assistant. """
            request_shutdown.set()

        self.services.register(
            DOMAIN, SERVICE_HOMEASSISTANT_STOP, stop_homeassistant)

        if os.name != "nt":
            try:
                signal.signal(signal.SIGQUIT, stop_homeassistant)
            except ValueError:
                _LOGGER.warning(
                    'Could not bind to SIGQUIT. Are you running in a thread?')

        while not request_shutdown.isSet():
            try:
                time.sleep(1)
            except KeyboardInterrupt:
                break

        self.stop()

    def stop(self):
        """ Stops Home Assistant and shuts down all threads. """
        _LOGGER.info("Stopping")

        self.bus.fire(EVENT_HOMEASSISTANT_STOP)

        # Wait till all responses to homeassistant_stop are done
        self.pool.block_till_done()

        self.pool.stop()

    def track_point_in_time(self, action, point_in_time):
        """Deprecated method as of 8/4/2015 to track point in time."""
        _LOGGER.warning(
            'hass.track_point_in_time is deprecated. '
            'Please use homeassistant.helpers.event.track_point_in_time')
        import homeassistant.helpers.event as helper
        helper.track_point_in_time(self, action, point_in_time)

    def track_point_in_utc_time(self, action, point_in_time):
        """Deprecated method as of 8/4/2015 to track point in UTC time."""
        _LOGGER.warning(
            'hass.track_point_in_utc_time is deprecated. '
            'Please use homeassistant.helpers.event.track_point_in_utc_time')
        import homeassistant.helpers.event as helper
        helper.track_point_in_utc_time(self, action, point_in_time)

    def track_utc_time_change(self, action,
                              year=None, month=None, day=None,
                              hour=None, minute=None, second=None):
        """Deprecated method as of 8/4/2015 to track UTC time change."""
        # pylint: disable=too-many-arguments
        _LOGGER.warning(
            'hass.track_utc_time_change is deprecated. '
            'Please use homeassistant.helpers.event.track_utc_time_change')
        import homeassistant.helpers.event as helper
        helper.track_utc_time_change(self, action, year, month, day, hour,
                                     minute, second)

    def track_time_change(self, action,
                          year=None, month=None, day=None,
                          hour=None, minute=None, second=None, utc=False):
        """Deprecated method as of 8/4/2015 to track time change."""
        # pylint: disable=too-many-arguments
        _LOGGER.warning(
            'hass.track_time_change is deprecated. '
            'Please use homeassistant.helpers.event.track_time_change')
        import homeassistant.helpers.event as helper
        helper.track_time_change(self, action, year, month, day, hour,
                                 minute, second)


class JobPriority(util.OrderedEnum):
    """ Provides priorities for bus events. """
    # pylint: disable=no-init,too-few-public-methods

    EVENT_CALLBACK = 0
    EVENT_SERVICE = 1
    EVENT_STATE = 2
    EVENT_TIME = 3
    EVENT_DEFAULT = 4

    @staticmethod
    def from_event_type(event_type):
        """ Returns a priority based on event type. """
        if event_type == EVENT_TIME_CHANGED:
            return JobPriority.EVENT_TIME
        elif event_type == EVENT_STATE_CHANGED:
            return JobPriority.EVENT_STATE
        elif event_type == EVENT_CALL_SERVICE:
            return JobPriority.EVENT_SERVICE
        elif event_type == EVENT_SERVICE_EXECUTED:
            return JobPriority.EVENT_CALLBACK
        else:
            return JobPriority.EVENT_DEFAULT


class EventOrigin(enum.Enum):
    """ Distinguish between origin of event. """
    # pylint: disable=no-init,too-few-public-methods

    local = "LOCAL"
    remote = "REMOTE"

    def __str__(self):
        return self.value


# pylint: disable=too-few-public-methods
class Event(object):
    """ Represents an event within the Bus. """

    __slots__ = ['event_type', 'data', 'origin', 'time_fired']

    def __init__(self, event_type, data=None, origin=EventOrigin.local,
                 time_fired=None):
        self.event_type = event_type
        self.data = data or {}
        self.origin = origin
        self.time_fired = date_util.strip_microseconds(
            time_fired or date_util.utcnow())

    def as_dict(self):
        """ Returns a dict representation of this Event. """
        return {
            'event_type': self.event_type,
            'data': dict(self.data),
            'origin': str(self.origin),
            'time_fired': date_util.datetime_to_str(self.time_fired),
        }

    def __repr__(self):
        # pylint: disable=maybe-no-member
        if self.data:
            return "<Event {}[{}]: {}>".format(
                self.event_type, str(self.origin)[0],
                util.repr_helper(self.data))
        else:
            return "<Event {}[{}]>".format(self.event_type,
                                           str(self.origin)[0])

    def __eq__(self, other):
        return (self.__class__ == other.__class__ and
                self.event_type == other.event_type and
                self.data == other.data and
                self.origin == other.origin and
                self.time_fired == other.time_fired)


class EventBus(object):
    """ Class that allows different components to communicate via services
    and events.
    """

    def __init__(self, pool=None):
        self._listeners = {}
        self._lock = threading.Lock()
        self._pool = pool or create_worker_pool()

    @property
    def listeners(self):
        """ Dict with events that is being listened for and the number
        of listeners.
        """
        with self._lock:
            return {key: len(self._listeners[key])
                    for key in self._listeners}

    def fire(self, event_type, event_data=None, origin=EventOrigin.local):
        """ Fire an event. """
        if not self._pool.running:
            raise HomeAssistantError('Home Assistant has shut down.')

        with self._lock:
            # Copy the list of the current listeners because some listeners
            # remove themselves as a listener while being executed which
            # causes the iterator to be confused.
            get = self._listeners.get
            listeners = get(MATCH_ALL, []) + get(event_type, [])

            event = Event(event_type, event_data, origin)

            if event_type != EVENT_TIME_CHANGED:
                _LOGGER.info("Bus:Handling %s", event)

            if not listeners:
                return

            job_priority = JobPriority.from_event_type(event_type)

            for func in listeners:
                self._pool.add_job(job_priority, (func, event))

    def listen(self, event_type, listener):
        """ Listen for all events or events of a specific type.

        To listen to all events specify the constant ``MATCH_ALL``
        as event_type.
        """
        with self._lock:
            if event_type in self._listeners:
                self._listeners[event_type].append(listener)
            else:
                self._listeners[event_type] = [listener]

    def listen_once(self, event_type, listener):
        """ Listen once for event of a specific type.

        To listen to all events specify the constant ``MATCH_ALL``
        as event_type.

        Returns registered listener that can be used with remove_listener.
        """
        @ft.wraps(listener)
        def onetime_listener(event):
            """ Removes listener from eventbus and then fires listener. """
            if hasattr(onetime_listener, 'run'):
                return
            # Set variable so that we will never run twice.
            # Because the event bus might have to wait till a thread comes
            # available to execute this listener it might occur that the
            # listener gets lined up twice to be executed.
            # This will make sure the second time it does nothing.
            onetime_listener.run = True

            self.remove_listener(event_type, onetime_listener)

            listener(event)

        self.listen(event_type, onetime_listener)

        return onetime_listener

    def remove_listener(self, event_type, listener):
        """ Removes a listener of a specific event_type. """
        with self._lock:
            try:
                self._listeners[event_type].remove(listener)

                # delete event_type list if empty
                if not self._listeners[event_type]:
                    self._listeners.pop(event_type)

            except (KeyError, ValueError):
                # KeyError is key event_type listener did not exist
                # ValueError if listener did not exist within event_type
                pass


class State(object):
    """
    Object to represent a state within the state machine.

    entity_id: the entity that is represented.
    state: the state of the entity
    attributes: extra information on entity and state
    last_changed: last time the state was changed, not the attributes.
    last_updated: last time this object was updated.
    """

    __slots__ = ['entity_id', 'state', 'attributes',
                 'last_changed', 'last_updated']

    # pylint: disable=too-many-arguments
    def __init__(self, entity_id, state, attributes=None, last_changed=None,
                 last_updated=None):
        if not ENTITY_ID_PATTERN.match(entity_id):
            raise InvalidEntityFormatError((
                "Invalid entity id encountered: {}. "
                "Format should be <domain>.<object_id>").format(entity_id))

        self.entity_id = entity_id.lower()
        self.state = state
        self.attributes = attributes or {}
        self.last_updated = date_util.strip_microseconds(
            last_updated or date_util.utcnow())

        # Strip microsecond from last_changed else we cannot guarantee
        # state == State.from_dict(state.as_dict())
        # This behavior occurs because to_dict uses datetime_to_str
        # which does not preserve microseconds
        self.last_changed = date_util.strip_microseconds(
            last_changed or self.last_updated)

    @property
    def domain(self):
        """ Returns domain of this state. """
        return util.split_entity_id(self.entity_id)[0]

    @property
    def object_id(self):
        """ Returns object_id of this state. """
        return util.split_entity_id(self.entity_id)[1]

    @property
    def name(self):
        """ Name to represent this state. """
        return (
            self.attributes.get(ATTR_FRIENDLY_NAME) or
            self.object_id.replace('_', ' '))

    def copy(self):
        """ Creates a copy of itself. """
        return State(self.entity_id, self.state,
                     dict(self.attributes), self.last_changed)

    def as_dict(self):
        """ Converts State to a dict to be used within JSON.
        Ensures: state == State.from_dict(state.as_dict()) """

        return {'entity_id': self.entity_id,
                'state': self.state,
                'attributes': self.attributes,
                'last_changed': date_util.datetime_to_str(self.last_changed),
                'last_updated': date_util.datetime_to_str(self.last_updated)}

    @classmethod
    def from_dict(cls, json_dict):
        """ Static method to create a state from a dict.
        Ensures: state == State.from_json_dict(state.to_json_dict()) """

        if not (json_dict and
                'entity_id' in json_dict and
                'state' in json_dict):
            return None

        last_changed = json_dict.get('last_changed')

        if last_changed:
            last_changed = date_util.str_to_datetime(last_changed)

        last_updated = json_dict.get('last_updated')

        if last_updated:
            last_updated = date_util.str_to_datetime(last_updated)

        return cls(json_dict['entity_id'], json_dict['state'],
                   json_dict.get('attributes'), last_changed, last_updated)

    def __eq__(self, other):
        return (self.__class__ == other.__class__ and
                self.entity_id == other.entity_id and
                self.state == other.state and
                self.attributes == other.attributes)

    def __repr__(self):
        attr = "; {}".format(util.repr_helper(self.attributes)) \
               if self.attributes else ""

        return "<state {}={}{} @ {}>".format(
            self.entity_id, self.state, attr,
            date_util.datetime_to_local_str(self.last_changed))


class StateMachine(object):
    """ Helper class that tracks the state of different entities. """

    def __init__(self, bus):
        self._states = {}
        self._bus = bus
        self._lock = threading.Lock()

    def entity_ids(self, domain_filter=None):
        """ List of entity ids that are being tracked. """
        if domain_filter is None:
            return list(self._states.keys())

        domain_filter = domain_filter.lower()

        return [state.entity_id for key, state
                in self._states.items()
                if util.split_entity_id(key)[0] == domain_filter]

    def all(self):
        """ Returns a list of all states. """
        with self._lock:
            return [state.copy() for state in self._states.values()]

    def get(self, entity_id):
        """ Returns the state of the specified entity. """
        state = self._states.get(entity_id.lower())

        # Make a copy so people won't mutate the state
        return state.copy() if state else None

    def is_state(self, entity_id, state):
        """ Returns True if entity exists and is specified state. """
        entity_id = entity_id.lower()

        return (entity_id in self._states and
                self._states[entity_id].state == state)

    def remove(self, entity_id):
        """ Removes an entity from the state machine.

        Returns boolean to indicate if an entity was removed. """
        entity_id = entity_id.lower()

        with self._lock:
            return self._states.pop(entity_id, None) is not None

    def set(self, entity_id, new_state, attributes=None):
        """ Set the state of an entity, add entity if it does not exist.

        Attributes is an optional dict to specify attributes of this state.

        If you just update the attributes and not the state, last changed will
        not be affected.
        """
        entity_id = entity_id.lower()
        new_state = str(new_state)
        attributes = attributes or {}

        with self._lock:
            old_state = self._states.get(entity_id)

            is_existing = old_state is not None
            same_state = is_existing and old_state.state == new_state
            same_attr = is_existing and old_state.attributes == attributes

            if same_state and same_attr:
                return

            # If state did not exist or is different, set it
            last_changed = old_state.last_changed if same_state else None

            state = State(entity_id, new_state, attributes, last_changed)
            self._states[entity_id] = state

            event_data = {'entity_id': entity_id, 'new_state': state}

            if old_state:
                event_data['old_state'] = old_state

            self._bus.fire(EVENT_STATE_CHANGED, event_data)

    def track_change(self, entity_ids, action, from_state=None, to_state=None):
        """
        DEPRECATED AS OF 8/4/2015
        """
        _LOGGER.warning(
            'hass.states.track_change is deprecated. '
            'Use homeassistant.helpers.event.track_state_change instead.')
        import homeassistant.helpers.event as helper
        helper.track_state_change(_MockHA(self._bus), entity_ids, action,
                                  from_state, to_state)


# pylint: disable=too-few-public-methods
class ServiceCall(object):
    """ Represents a call to a service. """

    __slots__ = ['domain', 'service', 'data']

    def __init__(self, domain, service, data=None):
        self.domain = domain
        self.service = service
        self.data = data or {}

    def __repr__(self):
        if self.data:
            return "<ServiceCall {}.{}: {}>".format(
                self.domain, self.service, util.repr_helper(self.data))
        else:
            return "<ServiceCall {}.{}>".format(self.domain, self.service)


class ServiceRegistry(object):
    """ Offers services over the eventbus. """

    def __init__(self, bus, pool=None):
        self._services = {}
        self._lock = threading.Lock()
        self._pool = pool or create_worker_pool()
        self._bus = bus
        self._cur_id = 0
        bus.listen(EVENT_CALL_SERVICE, self._event_to_service_call)

    @property
    def services(self):
        """ Dict with per domain a list of available services. """
        with self._lock:
            return {domain: list(self._services[domain].keys())
                    for domain in self._services}

    def has_service(self, domain, service):
        """ Returns True if specified service exists. """
        return service in self._services.get(domain, [])

    def register(self, domain, service, service_func):
        """ Register a service. """
        with self._lock:
            if domain in self._services:
                self._services[domain][service] = service_func
            else:
                self._services[domain] = {service: service_func}

            self._bus.fire(
                EVENT_SERVICE_REGISTERED,
                {ATTR_DOMAIN: domain, ATTR_SERVICE: service})

    def call(self, domain, service, service_data=None, blocking=False):
        """
        Calls specified service.
        Specify blocking=True to wait till service is executed.
        Waits a maximum of SERVICE_CALL_LIMIT.

        If blocking = True, will return boolean if service executed
        succesfully within SERVICE_CALL_LIMIT.

        This method will fire an event to call the service.
        This event will be picked up by this ServiceRegistry and any
        other ServiceRegistry that is listening on the EventBus.

        Because the service is sent as an event you are not allowed to use
        the keys ATTR_DOMAIN and ATTR_SERVICE in your service_data.
        """
        call_id = self._generate_unique_id()
        event_data = service_data or {}
        event_data[ATTR_DOMAIN] = domain
        event_data[ATTR_SERVICE] = service
        event_data[ATTR_SERVICE_CALL_ID] = call_id

        if blocking:
            executed_event = threading.Event()

            def service_executed(call):
                """
                Called when a service is executed.
                Will set the event if matches our service call.
                """
                if call.data[ATTR_SERVICE_CALL_ID] == call_id:
                    executed_event.set()

            self._bus.listen(EVENT_SERVICE_EXECUTED, service_executed)

        self._bus.fire(EVENT_CALL_SERVICE, event_data)

        if blocking:
            success = executed_event.wait(SERVICE_CALL_LIMIT)
            self._bus.remove_listener(
                EVENT_SERVICE_EXECUTED, service_executed)
            return success

    def _event_to_service_call(self, event):
        """ Calls a service from an event. """
        service_data = dict(event.data)
        domain = service_data.pop(ATTR_DOMAIN, None)
        service = service_data.pop(ATTR_SERVICE, None)

        if not self.has_service(domain, service):
            return

        service_handler = self._services[domain][service]
        service_call = ServiceCall(domain, service, service_data)

        # Add a job to the pool that calls _execute_service
        self._pool.add_job(JobPriority.EVENT_SERVICE,
                           (self._execute_service,
                            (service_handler, service_call)))

    def _execute_service(self, service_and_call):
        """ Executes a service and fires a SERVICE_EXECUTED event. """
        service, call = service_and_call
        service(call)

        if ATTR_SERVICE_CALL_ID in call.data:
            self._bus.fire(
                EVENT_SERVICE_EXECUTED,
                {ATTR_SERVICE_CALL_ID: call.data[ATTR_SERVICE_CALL_ID]})

    def _generate_unique_id(self):
        """ Generates a unique service call id. """
        self._cur_id += 1
        return "{}-{}".format(id(self), self._cur_id)


class Config(object):
    """ Configuration settings for Home Assistant. """

    # pylint: disable=too-many-instance-attributes
    def __init__(self):
        self.latitude = None
        self.longitude = None
        self.temperature_unit = None
        self.location_name = None
        self.time_zone = None

        # List of loaded components
        self.components = []

        # Remote.API object pointing at local API
        self.api = None

        # Directory that holds the configuration
        self.config_dir = get_default_config_dir()

    def path(self, *path):
        """ Returns path to the file within the config dir. """
        return os.path.join(self.config_dir, *path)

    def temperature(self, value, unit):
        """ Converts temperature to user preferred unit if set. """
        if not (unit in (TEMP_CELCIUS, TEMP_FAHRENHEIT) and
                self.temperature_unit and unit != self.temperature_unit):
            return value, unit

        try:
            temp = float(value)
        except ValueError:  # Could not convert value to float
            return value, unit

        return (
            round(temp_helper.convert(temp, unit, self.temperature_unit), 1),
            self.temperature_unit)

    def as_dict(self):
        """ Converts config to a dictionary. """
        time_zone = self.time_zone or date_util.UTC

        return {
            'latitude': self.latitude,
            'longitude': self.longitude,
            'temperature_unit': self.temperature_unit,
            'location_name': self.location_name,
            'time_zone': time_zone.zone,
            'components': self.components,
        }


def create_timer(hass, interval=TIMER_INTERVAL):
    """ Creates a timer. Timer will start on HOMEASSISTANT_START. """
    # We want to be able to fire every time a minute starts (seconds=0).
    # We want this so other modules can use that to make sure they fire
    # every minute.
    assert 60 % interval == 0, "60 % TIMER_INTERVAL should be 0!"

    def timer():
        """Send an EVENT_TIME_CHANGED on interval."""
        stop_event = threading.Event()

        def stop_timer(event):
            """Stop the timer."""
            stop_event.set()

        hass.bus.listen_once(EVENT_HOMEASSISTANT_STOP, stop_timer)

        _LOGGER.info("Timer:starting")

        last_fired_on_second = -1

        calc_now = date_util.utcnow

        while not stop_event.isSet():
            now = calc_now()

            # First check checks if we are not on a second matching the
            # timer interval. Second check checks if we did not already fire
            # this interval.
            if now.second % interval or \
               now.second == last_fired_on_second:

                # Sleep till it is the next time that we have to fire an event.
                # Aim for halfway through the second that fits TIMER_INTERVAL.
                # If TIMER_INTERVAL is 10 fire at .5, 10.5, 20.5, etc seconds.
                # This will yield the best results because time.sleep() is not
                # 100% accurate because of non-realtime OS's
                slp_seconds = interval - now.second % interval + \
                    .5 - now.microsecond/1000000.0

                time.sleep(slp_seconds)

                now = calc_now()

            last_fired_on_second = now.second

            # Event might have been set while sleeping
            if not stop_event.isSet():
                try:
                    hass.bus.fire(EVENT_TIME_CHANGED, {ATTR_NOW: now})
                except HomeAssistantError:
                    # HA raises error if firing event after it has shut down
                    break

    def start_timer(event):
        """Start the timer."""
        thread = threading.Thread(target=timer)
        thread.daemon = True
        thread.start()

    hass.bus.listen_once(EVENT_HOMEASSISTANT_START, start_timer)


def create_worker_pool(worker_count=None):
    """ Creates a worker pool to be used. """
    if worker_count is None:
        worker_count = MIN_WORKER_THREAD

    def job_handler(job):
        """ Called whenever a job is available to do. """
        try:
            func, arg = job
            func(arg)
        except Exception:  # pylint: disable=broad-except
            # Catch any exception our service/event_listener might throw
            # We do not want to crash our ThreadPool
            _LOGGER.exception("BusHandler:Exception doing job")

    def busy_callback(worker_count, current_jobs, pending_jobs_count):
        """ Callback to be called when the pool queue gets too big. """

        _LOGGER.warning(
            "WorkerPool:All %d threads are busy and %d jobs pending",
            worker_count, pending_jobs_count)

        for start, job in current_jobs:
            _LOGGER.warning("WorkerPool:Current job from %s: %s",
                            date_util.datetime_to_local_str(start), job)

    return util.ThreadPool(job_handler, worker_count, busy_callback)



"""
homeassistant.components.sensor.arduino
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Support for getting information from Arduino pins. Only analog pins are
supported.

Configuration:

sensor:
  platform: arduino
  pins:
    7:
      name: Door switch
      type: analog
    0:
      name: Brightness
      type: analog

Variables:

pins
*Required
An array specifying the digital pins to use on the Arduino board.

These are the variables for the pins array:

name
*Required
The name for the pin that will be used in the frontend.

type
*Required
The type of the pin: 'analog'.
"""
import logging

import homeassistant.components.arduino as arduino
from homeassistant.helpers.entity import Entity
from homeassistant.const import DEVICE_DEFAULT_NAME

DEPENDENCIES = ['arduino']

_LOGGER = logging.getLogger(__name__)


def setup_platform(hass, config, add_devices, discovery_info=None):
    """ Sets up the Arduino platform. """

    # Verify that Arduino board is present
    if arduino.BOARD is None:
        _LOGGER.error('A connection has not been made to the Arduino board.')
        return False

    sensors = []
    pins = config.get('pins')
    for pinnum, pin in pins.items():
        if pin.get('name'):
            sensors.append(ArduinoSensor(pin.get('name'),
                                         pinnum,
                                         'analog'))
    add_devices(sensors)


class ArduinoSensor(Entity):
    """ Represents an Arduino Sensor. """
    def __init__(self, name, pin, pin_type):
        self._pin = pin
        self._name = name or DEVICE_DEFAULT_NAME
        self.pin_type = pin_type
        self.direction = 'in'
        self._value = None

        arduino.BOARD.set_mode(self._pin, self.direction, self.pin_type)

    @property
    def state(self):
        """ Returns the state of the sensor. """
        return self._value

    @property
    def name(self):
        """ Get the name of the sensor. """
        return self._name

    def update(self):
        """ Get the latest value from the pin. """
        self._value = arduino.BOARD.get_analog_inputs()[self._pin][1]



"""Color util methods."""


# Taken from: http://www.cse.unr.edu/~quiroz/inc/colortransforms.py
# License: Code is given as is. Use at your own risk and discretion.
# pylint: disable=invalid-name
def color_RGB_to_xy(R, G, B):
    """ Convert from RGB color to XY color. """
    if R + G + B == 0:
        return 0, 0

    var_R = (R / 255.)
    var_G = (G / 255.)
    var_B = (B / 255.)

    if var_R > 0.04045:
        var_R = ((var_R + 0.055) / 1.055) ** 2.4
    else:
        var_R /= 12.92

    if var_G > 0.04045:
        var_G = ((var_G + 0.055) / 1.055) ** 2.4
    else:
        var_G /= 12.92

    if var_B > 0.04045:
        var_B = ((var_B + 0.055) / 1.055) ** 2.4
    else:
        var_B /= 12.92

    var_R *= 100
    var_G *= 100
    var_B *= 100

    # Observer. = 2 deg, Illuminant = D65
    X = var_R * 0.4124 + var_G * 0.3576 + var_B * 0.1805
    Y = var_R * 0.2126 + var_G * 0.7152 + var_B * 0.0722
    Z = var_R * 0.0193 + var_G * 0.1192 + var_B * 0.9505

    # Convert XYZ to xy, see CIE 1931 color space on wikipedia
    return X / (X + Y + Z), Y / (X + Y + Z)



__author__ = 'jamespcole'

import json
import time

import requests

"""
Vera Controller Python API
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

This lib is designed to simplify communication with Vera Z-Wave controllers
"""

class VeraController(object):

	temperature_units = 'C'

	def __init__(self, baseUrl):
		self.BASE_URL = baseUrl
		self.devices = []

	def get_simple_devices_info(self):

		simpleRequestUrl = self.BASE_URL + "/data_request?id=sdata"
		j = requests.get(simpleRequestUrl).json()

		if j.get('temperature'):
			self.temperature_units = j.get('temperature')

		self.categories = {}
		cats = j.get('categories')

		for cat in cats:
			self.categories[cat.get('id')] = cat.get('name')

		self.device_id_map = {}

		devs = j.get('devices')
		for dev in devs:
			dev['categoryName'] = self.categories.get(dev.get('category'))
			self.device_id_map[dev.get('id')] = dev

	#get list of connected devices, the categoryFilter param can be either a string or array of strings
	def get_devices(self, categoryFilter=''):

		# the Vera rest API is a bit rough so we need to make 2 calls to get all the info e need
		self.get_simple_devices_info()

		arequestUrl = self.BASE_URL + "/data_request?id=status&output_format=json"
		j = requests.get(arequestUrl).json()

		self.devices = []
		items = j.get('devices')

		for item in items:
			item['deviceInfo'] = self.device_id_map.get(item.get('id'))
			if item.get('deviceInfo') and item.get('deviceInfo').get('categoryName') == 'Switch':
				self.devices.append(VeraSwitch(item, self))
			elif item.get('deviceInfo') and item.get('deviceInfo').get('categoryName') == 'On/Off Switch':
				self.devices.append(VeraSwitch(item, self))
			elif item.get('deviceInfo') and item.get('deviceInfo').get('categoryName') == 'Temperature Sensor':
				self.devices.append(VeraSensor(item, self))
			elif item.get('deviceInfo') and item.get('deviceInfo').get('categoryName') == 'Sensor':
				sensor = VeraSensor(item, self)
				self.devices.append(sensor)
				if sensor.is_armable:
					armable = VeraArmableDevice(item, self)
					armable.category = 'Armable Sensor'
					self.devices.append(armable)
			elif item.get('deviceInfo') and item.get('deviceInfo').get('categoryName') == 'Light Sensor':
				self.devices.append(VeraSensor(item, self))
			else:
				self.devices.append(VeraDevice(item, self))

		if categoryFilter == '':
			return self.devices
		else:
			filterCategories = []
			if isinstance(categoryFilter, str):
				filterCategories.append(categoryFilter)
			else:
				filterCategories = categoryFilter

			devices = []
			for item in self.devices:
				if item.category in filterCategories:
					devices.append(item)
			return devices



class VeraDevice(object):

	def __init__(self, aJSonObj, veraController):
		self.jsonState = aJSonObj
		self.deviceId = self.jsonState.get('id')
		self.veraController = veraController
		self.name = ''
		if self.jsonState.get('deviceInfo'):
			self.category = self.jsonState.get('deviceInfo').get('categoryName')
			self.name = self.jsonState.get('deviceInfo').get('name')
		else:
			self.category = ''

		if not self.name:
			if self.category:
				self.name = 'Vera ' + self.category + ' ' + str(self.deviceId)
			else:
				self.name = 'Vera Device ' + str(self.deviceId)


	def set_value(self, name, value):
		for item in self.jsonState.get('states'):
			if item.get('variable') == name:
				serviceName = item.get('service')
				payload = {'id': 'lu_action', 'output_format': 'json', 'DeviceNum': self.deviceId, 'serviceId': serviceName, 'action': 'Set' + name, 'new' + name + 'Value': value}
				requestUrl = self.veraController.BASE_URL + "/data_request"
				r = requests.get(requestUrl, params=payload)
				item['value'] = value

	def get_value(self, name):
		for item in self.jsonState.get('states'):
			if item.get('variable') == name:
				return item.get('value')
		return None

	def refresh_value(self, name):
		for item in self.jsonState.get('states'):
			if item.get('variable') == name:
				serviceName = item.get('service')
				payload = {'id': 'variableget', 'output_format': 'json', 'DeviceNum': self.deviceId, 'serviceId': serviceName, 'Variable': name}
				requestUrl = self.veraController.BASE_URL + "/data_request"
				r = requests.get(requestUrl, params=payload)
				item['value'] = r.text
				return item.get('value')
		return None

	@property
	def is_armable(self):
		if self.get_value('Armed') is not None:
			return True
		else:
			return False

	@property
	def is_trippable(self):
		if self.get_value('Tripped') is not None:
			return True
		else:
			return False

	@property
	def has_battery(self):
		if self.get_value('BatteryLevel') is not None:
			return True
		else:
			return False

	@property
	def battery_level(self):
		return self.refresh_value('BatteryLevel')

	@property
	def vera_device_id(self):
		return self.deviceId


class VeraSwitch(VeraDevice):

	def __init__(self, aJSonObj, veraController):
		super().__init__(aJSonObj, veraController)

	def switch_on(self):
		self.set_value('Target', 1)

	def switch_off(self):
		self.set_value('Target', 0)

	def is_switched_on(self):
		self.refresh_value('Status')
		val = self.get_value('Status')
		if val == '1':
			return True
		else:
			return False

class VeraArmableDevice(VeraSwitch):

	def __init__(self, aJSonObj, veraController):
		super().__init__(aJSonObj, veraController)

	def switch_on(self):
		self.set_value('Armed', 1)

	def switch_off(self):
		self.set_value('Armed', 0)

	def is_switched_on(self):
		self.refresh_value('Armed')
		val = self.get_value('Armed')
		if val == '1':
			return True
		else:
			return False


class VeraSensor(VeraDevice):

	def __init__(self, aJSonObj, veraController):
		super().__init__(aJSonObj, veraController)

	def switch_on(self):
		self.set_value('Target', 1)

	def switch_off(self):
		self.set_value('Target', 0)

	def is_switched_on(self):
		self.refresh_value('Status')
		val = self.get_value('Status')
		if val == '1':
			return True
		else:
			return False


"""
homeassistant.components.notify.instapush
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Instapush notification service.

Configuration:

To use the Instapush notifier you will need to add something like the following
to your config/configuration.yaml

notify:
  platform: instapush
  api_key: YOUR_APP_KEY
  app_secret: YOUR_APP_SECRET
  event: YOUR_EVENT
  tracker: YOUR_TRACKER

VARIABLES:

api_key
*Required
To retrieve this value log into your account at https://instapush.im and go
to 'APPS', choose an app, and check 'Basic Info'.

app_secret
*Required
To get this value log into your account at https://instapush.im and go to
'APPS'. The 'Application ID' can be found under 'Basic Info'.

event
*Required
To retrieve a valid event log into your account at https://instapush.im and go
to 'APPS'. If you have no events to use with Home Assistant, create one event
for your app.

tracker
*Required
To retrieve the tracker value log into your account at https://instapush.im and
go to 'APPS', choose the app, and check the event entries.

Example usage of Instapush if you have an event 'notification' and a tracker
'home-assistant'.

curl -X POST \
    -H "x-instapush-appid: YOUR_APP_KEY" \
    -H "x-instapush-appsecret: YOUR_APP_SECRET" \
    -H "Content-Type: application/json" \
      -d '{"event":"notification","trackers":{"home-assistant":"Switch 1"}}' \
    https://api.instapush.im/v1/post

Details for the API : https://instapush.im/developer/rest

"""
import logging
import json

from homeassistant.helpers import validate_config
from homeassistant.components.notify import (
    DOMAIN, ATTR_TITLE, BaseNotificationService)
from homeassistant.const import CONF_API_KEY

_LOGGER = logging.getLogger(__name__)
_RESOURCE = 'https://api.instapush.im/v1/'


def get_service(hass, config):
    """ Get the instapush notification service. """

    if not validate_config(config,
                           {DOMAIN: [CONF_API_KEY,
                                     'app_secret',
                                     'event',
                                     'tracker']},
                           _LOGGER):
        return None

    try:
        import requests

    except ImportError:
        _LOGGER.exception(
            "Unable to import requests. "
            "Did you maybe not install the 'Requests' package?")

        return None

    # pylint: disable=unused-variable
    try:
        response = requests.get(_RESOURCE)

    except requests.ConnectionError:
        _LOGGER.error(
            "Connection error "
            "Please check if https://instapush.im is available.")

        return None

    instapush = requests.Session()
    headers = {'x-instapush-appid': config[DOMAIN][CONF_API_KEY],
               'x-instapush-appsecret': config[DOMAIN]['app_secret']}
    response = instapush.get(_RESOURCE + 'events/list',
                             headers=headers)

    try:
        if response.json()['error']:
            _LOGGER.error(response.json()['msg'])
    # pylint: disable=bare-except
    except:
        try:
            next(events for events in response.json()
                 if events['title'] == config[DOMAIN]['event'])
        except StopIteration:
            _LOGGER.error(
                "No event match your given value. "
                "Please create an event at https://instapush.im")
        else:
            return InstapushNotificationService(
                config[DOMAIN].get(CONF_API_KEY),
                config[DOMAIN]['app_secret'],
                config[DOMAIN]['event'],
                config[DOMAIN]['tracker']
                )


# pylint: disable=too-few-public-methods
class InstapushNotificationService(BaseNotificationService):
    """ Implements notification service for Instapush. """

    def __init__(self, api_key, app_secret, event, tracker):
        # pylint: disable=no-name-in-module, unused-variable
        from requests import Session

        self._api_key = api_key
        self._app_secret = app_secret
        self._event = event
        self._tracker = tracker
        self._headers = {
            'x-instapush-appid': self._api_key,
            'x-instapush-appsecret': self._app_secret,
            'Content-Type': 'application/json'}

        self.instapush = Session()

    def send_message(self, message="", **kwargs):
        """ Send a message to a user. """

        title = kwargs.get(ATTR_TITLE)

        data = {"event": self._event,
                "trackers": {self._tracker: title + " : " + message}}

        response = self.instapush.post(
            _RESOURCE + 'post',
            data=json.dumps(data),
            headers=self._headers)

        if response.json()['status'] == 401:
            _LOGGER.error(
                response.json()['msg'],
                "Please check your details at https://instapush.im/")



"""
homeassistant.components.browser
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Provides functionality to launch a webbrowser on the host machine.
"""

DOMAIN = "browser"
DEPENDENCIES = []

SERVICE_BROWSE_URL = "browse_url"


def setup(hass, config):
    """ Listen for browse_url events and open
        the url in the default webbrowser. """

    import webbrowser

    hass.services.register(DOMAIN, SERVICE_BROWSE_URL,
                           lambda service:
                           webbrowser.open(
                               service.data.get(
                                   'url', 'https://www.google.com')))

    return True



"""
homeassistant.components.mqtt
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
MQTT component, using paho-mqtt. This component needs a MQTT broker like
Mosquitto or Mosca. The Eclipse Foundation is running a public MQTT server
at iot.eclipse.org. If you prefer to use that one, keep in mind to adjust
the topic/client ID and that your messages are public.

Configuration:

To use MQTT you will need to add something like the following to your
config/configuration.yaml.

mqtt:
  broker: 127.0.0.1

Or, if you want more options:

mqtt:
  broker: 127.0.0.1
  port: 1883
  client_id: home-assistant-1
  keepalive: 60
  username: your_username
  password: your_secret_password

Variables:

broker
*Required
This is the IP address of your MQTT broker, e.g. 192.168.1.32.

port
*Optional
The network port to connect to. Default is 1883.

client_id
*Optional
Client ID that Home Assistant will use. Has to be unique on the server.
Default is a random generated one.

keepalive
*Optional
The keep alive in seconds for this client. Default is 60.
"""
import logging
import socket

from homeassistant.exceptions import HomeAssistantError
import homeassistant.util as util
from homeassistant.helpers import validate_config
from homeassistant.const import (
    EVENT_HOMEASSISTANT_START, EVENT_HOMEASSISTANT_STOP)

_LOGGER = logging.getLogger(__name__)

DOMAIN = "mqtt"

MQTT_CLIENT = None

DEFAULT_PORT = 1883
DEFAULT_KEEPALIVE = 60
DEFAULT_QOS = 0

SERVICE_PUBLISH = 'publish'
EVENT_MQTT_MESSAGE_RECEIVED = 'MQTT_MESSAGE_RECEIVED'

DEPENDENCIES = []
REQUIREMENTS = ['paho-mqtt==1.1']

CONF_BROKER = 'broker'
CONF_PORT = 'port'
CONF_CLIENT_ID = 'client_id'
CONF_KEEPALIVE = 'keepalive'
CONF_USERNAME = 'username'
CONF_PASSWORD = 'password'

ATTR_QOS = 'qos'
ATTR_TOPIC = 'topic'
ATTR_PAYLOAD = 'payload'


def publish(hass, topic, payload):
    """ Send an MQTT message. """
    data = {
        ATTR_TOPIC: topic,
        ATTR_PAYLOAD: payload,
    }
    hass.services.call(DOMAIN, SERVICE_PUBLISH, data)


def subscribe(hass, topic, callback, qos=0):
    """ Subscribe to a topic. """
    def mqtt_topic_subscriber(event):
        """ Match subscribed MQTT topic. """
        if _match_topic(topic, event.data[ATTR_TOPIC]):
            callback(event.data[ATTR_TOPIC], event.data[ATTR_PAYLOAD],
                     event.data[ATTR_QOS])

    hass.bus.listen(EVENT_MQTT_MESSAGE_RECEIVED, mqtt_topic_subscriber)

    if topic not in MQTT_CLIENT.topics:
        MQTT_CLIENT.subscribe(topic, qos)


def setup(hass, config):
    """ Get the MQTT protocol service. """

    if not validate_config(config, {DOMAIN: ['broker']}, _LOGGER):
        return False

    conf = config[DOMAIN]

    broker = conf[CONF_BROKER]
    port = util.convert(conf.get(CONF_PORT), int, DEFAULT_PORT)
    client_id = util.convert(conf.get(CONF_CLIENT_ID), str)
    keepalive = util.convert(conf.get(CONF_KEEPALIVE), int, DEFAULT_KEEPALIVE)
    username = util.convert(conf.get(CONF_USERNAME), str)
    password = util.convert(conf.get(CONF_PASSWORD), str)

    global MQTT_CLIENT
    try:
        MQTT_CLIENT = MQTT(hass, broker, port, client_id, keepalive, username,
                           password)
    except socket.error:
        _LOGGER.exception("Can't connect to the broker. "
                          "Please check your settings and the broker "
                          "itself.")
        return False

    def stop_mqtt(event):
        """ Stop MQTT component. """
        MQTT_CLIENT.stop()

    def start_mqtt(event):
        """ Launch MQTT component when Home Assistant starts up. """
        MQTT_CLIENT.start()
        hass.bus.listen_once(EVENT_HOMEASSISTANT_STOP, stop_mqtt)

    def publish_service(call):
        """ Handle MQTT publish service calls. """
        msg_topic = call.data.get(ATTR_TOPIC)
        payload = call.data.get(ATTR_PAYLOAD)
        if msg_topic is None or payload is None:
            return
        MQTT_CLIENT.publish(msg_topic, payload)

    hass.bus.listen_once(EVENT_HOMEASSISTANT_START, start_mqtt)

    hass.services.register(DOMAIN, SERVICE_PUBLISH, publish_service)

    return True


# This is based on one of the paho-mqtt examples:
# http://git.eclipse.org/c/paho/org.eclipse.paho.mqtt.python.git/tree/examples/sub-class.py
# pylint: disable=too-many-arguments
class MQTT(object):  # pragma: no cover
    """ Implements messaging service for MQTT. """
    def __init__(self, hass, broker, port, client_id, keepalive, username,
                 password):
        import paho.mqtt.client as mqtt

        self.hass = hass
        self._progress = {}
        self.topics = {}

        if client_id is None:
            self._mqttc = mqtt.Client()
        else:
            self._mqttc = mqtt.Client(client_id)
        if username is not None:
            self._mqttc.username_pw_set(username, password)
        self._mqttc.on_subscribe = self._mqtt_on_subscribe
        self._mqttc.on_unsubscribe = self._mqtt_on_unsubscribe
        self._mqttc.on_connect = self._mqtt_on_connect
        self._mqttc.on_message = self._mqtt_on_message
        self._mqttc.connect(broker, port, keepalive)

    def publish(self, topic, payload):
        """ Publish a MQTT message. """
        self._mqttc.publish(topic, payload)

    def unsubscribe(self, topic):
        """ Unsubscribe from topic. """
        result, mid = self._mqttc.unsubscribe(topic)
        _raise_on_error(result)
        self._progress[mid] = topic

    def start(self):
        """ Run the MQTT client. """
        self._mqttc.loop_start()

    def stop(self):
        """ Stop the MQTT client. """
        self._mqttc.loop_stop()

    def subscribe(self, topic, qos):
        """ Subscribe to a topic. """
        if topic in self.topics:
            return
        result, mid = self._mqttc.subscribe(topic, qos)
        _raise_on_error(result)
        self._progress[mid] = topic
        self.topics[topic] = None

    def _mqtt_on_connect(self, mqttc, obj, flags, result_code):
        """ On connect, resubscribe to all topics we were subscribed to. """
        old_topics = self.topics
        self._progress = {}
        self.topics = {}
        for topic, qos in old_topics.items():
            # qos is None if we were in process of subscribing
            if qos is not None:
                self._mqttc.subscribe(topic, qos)

    def _mqtt_on_subscribe(self, mqttc, obj, mid, granted_qos):
        """ Called when subscribe succesfull. """
        topic = self._progress.pop(mid, None)
        if topic is None:
            return
        self.topics[topic] = granted_qos

    def _mqtt_on_unsubscribe(self, mqttc, obj, mid, granted_qos):
        """ Called when subscribe succesfull. """
        topic = self._progress.pop(mid, None)
        if topic is None:
            return
        self.topics.pop(topic, None)

    def _mqtt_on_message(self, mqttc, obj, msg):
        """ Message callback """
        self.hass.bus.fire(EVENT_MQTT_MESSAGE_RECEIVED, {
            ATTR_TOPIC: msg.topic,
            ATTR_QOS: msg.qos,
            ATTR_PAYLOAD: msg.payload.decode('utf-8'),
        })


def _raise_on_error(result):  # pragma: no cover
    """ Raise error if error result. """
    if result != 0:
        raise HomeAssistantError('Error talking to MQTT: {}'.format(result))


def _match_topic(subscription, topic):
    """ Returns if topic matches subscription. """
    if subscription.endswith('#'):
        return (subscription[:-2] == topic or
                topic.startswith(subscription[:-1]))

    sub_parts = subscription.split('/')
    topic_parts = topic.split('/')

    return (len(sub_parts) == len(topic_parts) and
            all(a == b for a, b in zip(sub_parts, topic_parts) if a != '+'))



"""
homeassistant.components.notify.slack
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Slack platform for notify component.

Configuration:

To use the Slack notifier you will need to add something like the following
to your config/configuration.yaml

notify:
    platform: slack
    api_key: ABCDEFGHJKLMNOPQRSTUVXYZ
    default_channel: '#general'

Variables:

api_key
*Required
The slack API token to use for sending slack messages.
You can get your slack API token here https://api.slack.com/web?sudo=1

default_channel
*Required
The default channel to post to if no channel is explicitly specified when
sending the notification message.
"""
import logging

from homeassistant.helpers import validate_config
from homeassistant.components.notify import (
    DOMAIN, BaseNotificationService)
from homeassistant.const import CONF_API_KEY

REQUIREMENTS = ['slacker==0.6.8']
_LOGGER = logging.getLogger(__name__)


# pylint: disable=unused-variable
def get_service(hass, config):
    """ Get the slack notification service. """

    if not validate_config(config,
                           {DOMAIN: ['default_channel', CONF_API_KEY]},
                           _LOGGER):
        return None

    try:
        # pylint: disable=no-name-in-module, unused-variable
        from slacker import Error as SlackError

    except ImportError:
        _LOGGER.exception(
            "Unable to import slacker. "
            "Did you maybe not install the 'slacker.py' package?")

        return None

    try:
        api_token = config[DOMAIN].get(CONF_API_KEY)

        return SlackNotificationService(
            config[DOMAIN]['default_channel'],
            api_token)

    except SlackError as ex:
        _LOGGER.error(
            "Slack authentication failed")
        _LOGGER.exception(ex)


# pylint: disable=too-few-public-methods
class SlackNotificationService(BaseNotificationService):
    """ Implements notification service for Slack. """

    def __init__(self, default_channel, api_token):
        from slacker import Slacker
        self._default_channel = default_channel
        self._api_token = api_token
        self.slack = Slacker(self._api_token)
        self.slack.auth.test()

    def send_message(self, message="", **kwargs):
        """ Send a message to a user. """

        from slacker import Error as SlackError
        channel = kwargs.get('channel', self._default_channel)
        try:
            self.slack.chat.post_message(channel, message)
        except SlackError as ex:
            _LOGGER.exception("Could not send slack notification")
            _LOGGER.exception(ex)



"""
homeassistant.components.automation.time
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Offers time listening automation rules.
"""
from homeassistant.util import convert
from homeassistant.helpers.event import track_time_change

CONF_HOURS = "time_hours"
CONF_MINUTES = "time_minutes"
CONF_SECONDS = "time_seconds"


def register(hass, config, action):
    """ Listen for state changes based on `config`. """
    hours = convert(config.get(CONF_HOURS), int)
    minutes = convert(config.get(CONF_MINUTES), int)
    seconds = convert(config.get(CONF_SECONDS), int)

    def time_automation_listener(now):
        """ Listens for time changes and calls action. """
        action()

    track_time_change(hass, time_automation_listener,
                      hour=hours, minute=minutes, second=seconds)

    return True



"""
homeassistant.components.media_player.demo
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Demo implementation of the media player.

"""
from homeassistant.const import (
    STATE_PLAYING, STATE_PAUSED, STATE_OFF)

from homeassistant.components.media_player import (
    MediaPlayerDevice, YOUTUBE_COVER_URL_FORMAT,
    MEDIA_TYPE_VIDEO, MEDIA_TYPE_MUSIC, MEDIA_TYPE_TVSHOW,
    SUPPORT_PAUSE, SUPPORT_VOLUME_SET, SUPPORT_VOLUME_MUTE, SUPPORT_YOUTUBE,
    SUPPORT_TURN_ON, SUPPORT_TURN_OFF, SUPPORT_PREVIOUS_TRACK,
    SUPPORT_NEXT_TRACK)


# pylint: disable=unused-argument
def setup_platform(hass, config, add_devices, discovery_info=None):
    """ Sets up the cast platform. """
    add_devices([
        DemoYoutubePlayer(
            'Living Room', 'eyU3bRy2x44',
            '♥♥ The Best Fireplace Video (3 hours)'),
        DemoYoutubePlayer('Bedroom', 'kxopViU98Xo', 'Epic sax guy 10 hours'),
        DemoMusicPlayer(), DemoTVShowPlayer(),
    ])


YOUTUBE_PLAYER_SUPPORT = \
    SUPPORT_PAUSE | SUPPORT_VOLUME_SET | SUPPORT_VOLUME_MUTE | \
    SUPPORT_YOUTUBE | SUPPORT_TURN_ON | SUPPORT_TURN_OFF

MUSIC_PLAYER_SUPPORT = \
    SUPPORT_PAUSE | SUPPORT_VOLUME_SET | SUPPORT_VOLUME_MUTE | \
    SUPPORT_TURN_ON | SUPPORT_TURN_OFF

NETFLIX_PLAYER_SUPPORT = \
    SUPPORT_PAUSE | SUPPORT_TURN_ON | SUPPORT_TURN_OFF


class AbstractDemoPlayer(MediaPlayerDevice):
    """ Base class for demo media players. """
    # We only implement the methods that we support
    # pylint: disable=abstract-method

    def __init__(self, name):
        self._name = name
        self._player_state = STATE_PLAYING
        self._volume_level = 1.0
        self._volume_muted = False

    @property
    def should_poll(self):
        """ We will push an update after each command. """
        return False

    @property
    def name(self):
        """ Name of the media player. """
        return self._name

    @property
    def state(self):
        """ State of the player. """
        return self._player_state

    @property
    def volume_level(self):
        """ Volume level of the media player (0..1). """
        return self._volume_level

    @property
    def is_volume_muted(self):
        """ Boolean if volume is currently muted. """
        return self._volume_muted

    def turn_on(self):
        """ turn the media player on. """
        self._player_state = STATE_PLAYING
        self.update_ha_state()

    def turn_off(self):
        """ turn the media player off. """
        self._player_state = STATE_OFF
        self.update_ha_state()

    def mute_volume(self, mute):
        """ mute the volume. """
        self._volume_muted = mute
        self.update_ha_state()

    def set_volume_level(self, volume):
        """ set volume level, range 0..1. """
        self._volume_level = volume
        self.update_ha_state()

    def media_play(self):
        """ Send play commmand. """
        self._player_state = STATE_PLAYING
        self.update_ha_state()

    def media_pause(self):
        """ Send pause command. """
        self._player_state = STATE_PAUSED
        self.update_ha_state()


class DemoYoutubePlayer(AbstractDemoPlayer):
    """ A Demo media player that only supports YouTube. """
    # We only implement the methods that we support
    # pylint: disable=abstract-method

    def __init__(self, name, youtube_id=None, media_title=None):
        super().__init__(name)
        self.youtube_id = youtube_id
        self._media_title = media_title

    @property
    def media_content_id(self):
        """ Content ID of current playing media. """
        return self.youtube_id

    @property
    def media_content_type(self):
        """ Content type of current playing media. """
        return MEDIA_TYPE_VIDEO

    @property
    def media_duration(self):
        """ Duration of current playing media in seconds. """
        return 360

    @property
    def media_image_url(self):
        """ Image url of current playing media. """
        return YOUTUBE_COVER_URL_FORMAT.format(self.youtube_id)

    @property
    def media_title(self):
        """ Title of current playing media. """
        return self._media_title

    @property
    def app_name(self):
        """ Current running app. """
        return "YouTube"

    @property
    def supported_media_commands(self):
        """ Flags of media commands that are supported. """
        return YOUTUBE_PLAYER_SUPPORT

    def play_youtube(self, media_id):
        """ Plays a YouTube media. """
        self.youtube_id = media_id
        self._media_title = 'some YouTube video'
        self.update_ha_state()


class DemoMusicPlayer(AbstractDemoPlayer):
    """ A Demo media player that only supports YouTube. """
    # We only implement the methods that we support
    # pylint: disable=abstract-method

    tracks = [
        ('Technohead', 'I Wanna Be A Hippy (Flamman & Abraxas Radio Mix)'),
        ('Paul Elstak', 'Luv U More'),
        ('Dune', 'Hardcore Vibes'),
        ('Nakatomi', 'Children Of The Night'),
        ('Party Animals',
         'Have You Ever Been Mellow? (Flamman & Abraxas Radio Mix)'),
        ('Rob G.*', 'Ecstasy, You Got What I Need'),
        ('Lipstick', "I'm A Raver"),
        ('4 Tune Fairytales', 'My Little Fantasy (Radio Edit)'),
        ('Prophet', "The Big Boys Don't Cry"),
        ('Lovechild', 'All Out Of Love (DJ Weirdo & Sim Remix)'),
        ('Stingray & Sonic Driver', 'Cold As Ice (El Bruto Remix)'),
        ('Highlander', 'Hold Me Now (Bass-D & King Matthew Remix)'),
        ('Juggernaut', 'Ruffneck Rules Da Artcore Scene (12" Edit)'),
        ('Diss Reaction', 'Jiiieehaaaa '),
        ('Flamman And Abraxas', 'Good To Go (Radio Mix)'),
        ('Critical Mass', 'Dancing Together'),
        ('Charly Lownoise & Mental Theo',
         'Ultimate Sex Track (Bass-D & King Matthew Remix)'),
    ]

    def __init__(self):
        super().__init__('Walkman')
        self._cur_track = 0

    @property
    def media_content_id(self):
        """ Content ID of current playing media. """
        return 'bounzz-1'

    @property
    def media_content_type(self):
        """ Content type of current playing media. """
        return MEDIA_TYPE_MUSIC

    @property
    def media_duration(self):
        """ Duration of current playing media in seconds. """
        return 213

    @property
    def media_image_url(self):
        """ Image url of current playing media. """
        return 'https://graph.facebook.com/107771475912710/picture'

    @property
    def media_title(self):
        """ Title of current playing media. """
        return self.tracks[self._cur_track][1]

    @property
    def media_artist(self):
        """ Artist of current playing media. (Music track only) """
        return self.tracks[self._cur_track][0]

    @property
    def media_album_name(self):
        """ Album of current playing media. (Music track only) """
        # pylint: disable=no-self-use
        return "Bounzz"

    @property
    def media_track(self):
        """ Track number of current playing media. (Music track only) """
        return self._cur_track + 1

    @property
    def supported_media_commands(self):
        """ Flags of media commands that are supported. """
        support = MUSIC_PLAYER_SUPPORT

        if self._cur_track > 1:
            support |= SUPPORT_PREVIOUS_TRACK

        if self._cur_track < len(self.tracks)-1:
            support |= SUPPORT_NEXT_TRACK

        return support

    def media_previous_track(self):
        """ Send previous track command. """
        if self._cur_track > 0:
            self._cur_track -= 1
            self.update_ha_state()

    def media_next_track(self):
        """ Send next track command. """
        if self._cur_track < len(self.tracks)-1:
            self._cur_track += 1
            self.update_ha_state()


class DemoTVShowPlayer(AbstractDemoPlayer):
    """ A Demo media player that only supports YouTube. """
    # We only implement the methods that we support
    # pylint: disable=abstract-method

    def __init__(self):
        super().__init__('Lounge room')
        self._cur_episode = 1
        self._episode_count = 13

    @property
    def media_content_id(self):
        """ Content ID of current playing media. """
        return 'house-of-cards-1'

    @property
    def media_content_type(self):
        """ Content type of current playing media. """
        return MEDIA_TYPE_TVSHOW

    @property
    def media_duration(self):
        """ Duration of current playing media in seconds. """
        return 3600

    @property
    def media_image_url(self):
        """ Image url of current playing media. """
        return 'https://graph.facebook.com/HouseofCards/picture'

    @property
    def media_title(self):
        """ Title of current playing media. """
        return 'Chapter {}'.format(self._cur_episode)

    @property
    def media_series_title(self):
        """ Series title of current playing media. (TV Show only)"""
        return 'House of Cards'

    @property
    def media_season(self):
        """ Season of current playing media. (TV Show only) """
        return 1

    @property
    def media_episode(self):
        """ Episode of current playing media. (TV Show only) """
        return self._cur_episode

    @property
    def app_name(self):
        """ Current running app. """
        return "Netflix"

    @property
    def supported_media_commands(self):
        """ Flags of media commands that are supported. """
        support = NETFLIX_PLAYER_SUPPORT

        if self._cur_episode > 1:
            support |= SUPPORT_PREVIOUS_TRACK

        if self._cur_episode < self._episode_count:
            support |= SUPPORT_NEXT_TRACK

        return support

    def media_previous_track(self):
        """ Send previous track command. """
        if self._cur_episode > 1:
            self._cur_episode -= 1
            self.update_ha_state()

    def media_next_track(self):
        """ Send next track command. """
        if self._cur_episode < self._episode_count:
            self._cur_episode += 1
            self.update_ha_state()



"""
homeassistant.components.tracker
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Provides functionality to keep track of devices.
"""
import logging
import threading
import os
import csv
from datetime import timedelta

from homeassistant.helpers import validate_config
from homeassistant.helpers.entity import _OVERWRITE
import homeassistant.util as util
import homeassistant.util.dt as dt_util
from homeassistant.bootstrap import prepare_setup_platform

from homeassistant.helpers.event import track_utc_time_change
from homeassistant.const import (
    STATE_HOME, STATE_NOT_HOME, ATTR_ENTITY_PICTURE, ATTR_FRIENDLY_NAME,
    CONF_PLATFORM, DEVICE_DEFAULT_NAME)
from homeassistant.components import group

DOMAIN = "device_tracker"
DEPENDENCIES = []

SERVICE_DEVICE_TRACKER_RELOAD = "reload_devices_csv"

GROUP_NAME_ALL_DEVICES = 'all devices'
ENTITY_ID_ALL_DEVICES = group.ENTITY_ID_FORMAT.format('all_devices')

ENTITY_ID_FORMAT = DOMAIN + '.{}'

# After how much time do we consider a device not home if
# it does not show up on scans
TIME_DEVICE_NOT_FOUND = timedelta(minutes=3)

# Filename to save known devices to
KNOWN_DEVICES_FILE = "known_devices.csv"

CONF_SECONDS = "interval_seconds"

DEFAULT_CONF_SECONDS = 12

TRACK_NEW_DEVICES = "track_new_devices"

_LOGGER = logging.getLogger(__name__)


def is_on(hass, entity_id=None):
    """ Returns if any or specified device is home. """
    entity = entity_id or ENTITY_ID_ALL_DEVICES

    return hass.states.is_state(entity, STATE_HOME)


def setup(hass, config):
    """ Sets up the device tracker. """

    if not validate_config(config, {DOMAIN: [CONF_PLATFORM]}, _LOGGER):
        return False

    tracker_type = config[DOMAIN].get(CONF_PLATFORM)

    tracker_implementation = \
        prepare_setup_platform(hass, config, DOMAIN, tracker_type)

    if tracker_implementation is None:
        _LOGGER.error("Unknown device_tracker type specified: %s.",
                      tracker_type)

        return False

    device_scanner = tracker_implementation.get_scanner(hass, config)

    if device_scanner is None:
        _LOGGER.error("Failed to initialize device scanner: %s",
                      tracker_type)

        return False

    seconds = util.convert(config[DOMAIN].get(CONF_SECONDS), int,
                           DEFAULT_CONF_SECONDS)

    track_new_devices = config[DOMAIN].get(TRACK_NEW_DEVICES) or False
    _LOGGER.info("Tracking new devices: %s", track_new_devices)

    tracker = DeviceTracker(hass, device_scanner, seconds, track_new_devices)

    # We only succeeded if we got to parse the known devices file
    return not tracker.invalid_known_devices_file


class DeviceTracker(object):
    """ Class that tracks which devices are home and which are not. """

    def __init__(self, hass, device_scanner, seconds, track_new_devices):
        self.hass = hass

        self.device_scanner = device_scanner

        self.lock = threading.Lock()

        # Do we track new devices by default?
        self.track_new_devices = track_new_devices

        # Dictionary to keep track of known devices and devices we track
        self.tracked = {}
        self.untracked_devices = set()

        # Did we encounter an invalid known devices file
        self.invalid_known_devices_file = False

        # Wrap it in a func instead of lambda so it can be identified in
        # the bus by its __name__ attribute.
        def update_device_state(now):
            """ Triggers update of the device states. """
            self.update_devices(now)

        dev_group = group.Group(
            hass, GROUP_NAME_ALL_DEVICES, user_defined=False)

        def reload_known_devices_service(service):
            """ Reload known devices file. """
            self._read_known_devices_file()

            self.update_devices(dt_util.utcnow())

            dev_group.update_tracked_entity_ids(self.device_entity_ids)

        reload_known_devices_service(None)

        if self.invalid_known_devices_file:
            return

        seconds = range(0, 60, seconds)

        _LOGGER.info("Device tracker interval second=%s", seconds)
        track_utc_time_change(hass, update_device_state, second=seconds)

        hass.services.register(DOMAIN,
                               SERVICE_DEVICE_TRACKER_RELOAD,
                               reload_known_devices_service)

    @property
    def device_entity_ids(self):
        """ Returns a set containing all device entity ids
            that are being tracked. """
        return set(device['entity_id'] for device in self.tracked.values())

    def _update_state(self, now, device, is_home):
        """ Update the state of a device. """
        dev_info = self.tracked[device]

        if is_home:
            # Update last seen if at home
            dev_info['last_seen'] = now
        else:
            # State remains at home if it has been seen in the last
            # TIME_DEVICE_NOT_FOUND
            is_home = now - dev_info['last_seen'] < TIME_DEVICE_NOT_FOUND

        state = STATE_HOME if is_home else STATE_NOT_HOME

        # overwrite properties that have been set in the config file
        attr = dict(dev_info['state_attr'])
        attr.update(_OVERWRITE.get(dev_info['entity_id'], {}))

        self.hass.states.set(
            dev_info['entity_id'], state, attr)

    def update_devices(self, now):
        """ Update device states based on the found devices. """
        if not self.lock.acquire(False):
            return

        try:
            found_devices = set(dev.upper() for dev in
                                self.device_scanner.scan_devices())

            for device in self.tracked:
                is_home = device in found_devices

                self._update_state(now, device, is_home)

                if is_home:
                    found_devices.remove(device)

            # Did we find any devices that we didn't know about yet?
            new_devices = found_devices - self.untracked_devices

            if new_devices:
                if not self.track_new_devices:
                    self.untracked_devices.update(new_devices)

                self._update_known_devices_file(new_devices)
        finally:
            self.lock.release()

    # pylint: disable=too-many-branches
    def _read_known_devices_file(self):
        """ Parse and process the known devices file. """
        known_dev_path = self.hass.config.path(KNOWN_DEVICES_FILE)

        # Return if no known devices file exists
        if not os.path.isfile(known_dev_path):
            return

        self.lock.acquire()

        self.untracked_devices.clear()

        with open(known_dev_path) as inp:

            # To track which devices need an entity_id assigned
            need_entity_id = []

            # All devices that are still in this set after we read the CSV file
            # have been removed from the file and thus need to be cleaned up.
            removed_devices = set(self.tracked.keys())

            try:
                for row in csv.DictReader(inp):
                    device = row['device'].upper()

                    if row['track'] == '1':
                        if device in self.tracked:
                            # Device exists
                            removed_devices.remove(device)
                        else:
                            # We found a new device
                            need_entity_id.append(device)

                            self._track_device(device, row['name'])

                        # Update state_attr with latest from file
                        state_attr = {
                            ATTR_FRIENDLY_NAME: row['name']
                        }

                        if row['picture']:
                            state_attr[ATTR_ENTITY_PICTURE] = row['picture']

                        self.tracked[device]['state_attr'] = state_attr

                    else:
                        self.untracked_devices.add(device)

                # Remove existing devices that we no longer track
                for device in removed_devices:
                    entity_id = self.tracked[device]['entity_id']

                    _LOGGER.info("Removing entity %s", entity_id)

                    self.hass.states.remove(entity_id)

                    self.tracked.pop(device)

                self._generate_entity_ids(need_entity_id)

                if not self.tracked:
                    _LOGGER.warning(
                        "No devices to track. Please update %s.",
                        known_dev_path)

                _LOGGER.info("Loaded devices from %s", known_dev_path)

            except KeyError:
                self.invalid_known_devices_file = True

                _LOGGER.warning(
                    ("Invalid known devices file: %s. "
                     "We won't update it with new found devices."),
                    known_dev_path)

            finally:
                self.lock.release()

    def _update_known_devices_file(self, new_devices):
        """ Add new devices to known devices file. """
        if not self.invalid_known_devices_file:
            known_dev_path = self.hass.config.path(KNOWN_DEVICES_FILE)

            try:
                # If file does not exist we will write the header too
                is_new_file = not os.path.isfile(known_dev_path)

                with open(known_dev_path, 'a') as outp:
                    _LOGGER.info("Found %d new devices, updating %s",
                                 len(new_devices), known_dev_path)

                    writer = csv.writer(outp)

                    if is_new_file:
                        writer.writerow(("device", "name", "track", "picture"))

                    for device in new_devices:
                        # See if the device scanner knows the name
                        # else defaults to unknown device
                        name = self.device_scanner.get_device_name(device) or \
                            DEVICE_DEFAULT_NAME

                        track = 0
                        if self.track_new_devices:
                            self._track_device(device, name)
                            track = 1

                        writer.writerow((device, name, track, ""))

                if self.track_new_devices:
                    self._generate_entity_ids(new_devices)

            except IOError:
                _LOGGER.exception("Error updating %s with %d new devices",
                                  known_dev_path, len(new_devices))

    def _track_device(self, device, name):
        """
        Add a device to the list of tracked devices.
        Does not generate the entity id yet.
        """
        default_last_seen = dt_util.utcnow().replace(year=1990)

        self.tracked[device] = {
            'name': name,
            'last_seen': default_last_seen,
            'state_attr': {ATTR_FRIENDLY_NAME: name}
        }

    def _generate_entity_ids(self, need_entity_id):
        """ Generate entity ids for a list of devices. """
        # Setup entity_ids for the new devices
        used_entity_ids = [info['entity_id'] for device, info
                           in self.tracked.items()
                           if device not in need_entity_id]

        for device in need_entity_id:
            name = self.tracked[device]['name']

            entity_id = util.ensure_unique_string(
                ENTITY_ID_FORMAT.format(util.slugify(name)),
                used_entity_ids)

            used_entity_ids.append(entity_id)

            self.tracked[device]['entity_id'] = entity_id



"""Helpers to install PyPi packages."""
import os
import subprocess
import sys


def install_package(package, upgrade=False, target=None):
    """Install a package on PyPi. Accepts pip compatible package strings.
    Return boolean if install successfull."""
    # Not using 'import pip; pip.main([])' because it breaks the logger
    args = [sys.executable, '-m', 'pip', 'install', '--quiet', package]
    if upgrade:
        args.append('--upgrade')
    if target:
        args += ['--target', os.path.abspath(target)]
    try:
        return 0 == subprocess.call(args)
    except subprocess.SubprocessError:
        return False



"""
homeassistant.components.scene
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Allows users to set and activate scenes within Home Assistant.

A scene is a set of states that describe how you want certain entities to be.
For example, light A should be red with 100 brightness. Light B should be on.

A scene is active if all states of the scene match the real states.

If a scene is manually activated it will store the previous state of the
entities. These will be restored when the state is deactivated manually.

If one of the enties that are being tracked change state on its own, the
old state will not be restored when it is being deactivated.
"""
import logging
from collections import namedtuple

from homeassistant.core import State
from homeassistant.helpers.event import track_state_change
from homeassistant.helpers.entity import ToggleEntity
from homeassistant.helpers.entity_component import EntityComponent
from homeassistant.helpers.state import reproduce_state
from homeassistant.const import (
    ATTR_ENTITY_ID, STATE_OFF, STATE_ON, SERVICE_TURN_ON, SERVICE_TURN_OFF)

DOMAIN = 'scene'
DEPENDENCIES = ['group']

ATTR_ACTIVE_REQUESTED = "active_requested"

CONF_ENTITIES = "entities"

SceneConfig = namedtuple('SceneConfig', ['name', 'states'])


def setup(hass, config):
    """ Sets up scenes. """

    logger = logging.getLogger(__name__)

    scene_configs = config.get(DOMAIN)

    if not isinstance(scene_configs, list):
        logger.error('Scene config should be a list of scenes')
        return False

    component = EntityComponent(logger, DOMAIN, hass)

    component.add_entities(Scene(hass, _process_config(scene_config))
                           for scene_config in scene_configs)

    def handle_scene_service(service):
        """ Handles calls to the switch services. """
        target_scenes = component.extract_from_service(service)

        for scene in target_scenes:
            if service.service == SERVICE_TURN_ON:
                scene.turn_on()
            else:
                scene.turn_off()

    hass.services.register(DOMAIN, SERVICE_TURN_OFF, handle_scene_service)
    hass.services.register(DOMAIN, SERVICE_TURN_ON, handle_scene_service)

    return True


def _process_config(scene_config):
    """ Process passed in config into a format to work with. """
    name = scene_config.get('name')
    states = {}
    c_entities = dict(scene_config.get(CONF_ENTITIES, {}))

    for entity_id in c_entities:
        if isinstance(c_entities[entity_id], dict):
            state = c_entities[entity_id].pop('state', None)
            attributes = c_entities[entity_id]
        else:
            state = c_entities[entity_id]
            attributes = {}

        # YAML translates 'on' to a boolean
        # http://yaml.org/type/bool.html
        if isinstance(state, bool):
            state = STATE_ON if state else STATE_OFF
        else:
            state = str(state)

        states[entity_id.lower()] = State(entity_id, state, attributes)

    return SceneConfig(name, states)


class Scene(ToggleEntity):
    """ A scene is a group of entities and the states we want them to be. """

    def __init__(self, hass, scene_config):
        self.hass = hass
        self.scene_config = scene_config

        self.is_active = False
        self.prev_states = None
        self.ignore_updates = False

        track_state_change(
            self.hass, self.entity_ids, self.entity_state_changed)

        self.update()

    @property
    def should_poll(self):
        return False

    @property
    def name(self):
        return self.scene_config.name

    @property
    def is_on(self):
        return self.is_active

    @property
    def entity_ids(self):
        """ Entity IDs part of this scene. """
        return self.scene_config.states.keys()

    @property
    def state_attributes(self):
        """ Scene state attributes. """
        return {
            ATTR_ENTITY_ID: list(self.entity_ids),
            ATTR_ACTIVE_REQUESTED: self.prev_states is not None,
        }

    def turn_on(self):
        """ Activates scene. Tries to get entities into requested state. """
        self.prev_states = tuple(self.hass.states.get(entity_id)
                                 for entity_id in self.entity_ids)

        self._reproduce_state(self.scene_config.states.values())

    def turn_off(self):
        """ Deactivates scene and restores old states. """
        if self.prev_states:
            self._reproduce_state(self.prev_states)
            self.prev_states = None

    def entity_state_changed(self, entity_id, old_state, new_state):
        """ Called when an entity part of this scene changes state. """
        if self.ignore_updates:
            return

        # If new state is not what we expect, it can never be active
        if self._state_as_requested(new_state):
            self.update()
        else:
            self.is_active = False
            self.prev_states = None

        self.update_ha_state()

    def update(self):
        """
        Update if the scene is active.

        Will look at each requested state and see if the current entity
        has the same state and has at least the same attributes with the
        same values. The real state can have more attributes.
        """
        self.is_active = all(
            self._state_as_requested(self.hass.states.get(entity_id))
            for entity_id in self.entity_ids)

    def _state_as_requested(self, cur_state):
        """ Returns if given state is as requested. """
        state = self.scene_config.states.get(cur_state and cur_state.entity_id)

        return (cur_state is not None and state.state == cur_state.state and
                all(value == cur_state.attributes.get(key)
                    for key, value in state.attributes.items()))

    def _reproduce_state(self, states):
        """ Wraps reproduce state with Scence specific logic. """
        self.ignore_updates = True
        reproduce_state(self.hass, states, True)
        self.ignore_updates = False

        self.update_ha_state(True)



"""
homeassistant.components.sensor.dht
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Adafruit DHT temperature and humidity sensor.
You need a Python3 compatible version of the Adafruit_Python_DHT library
(e.g. https://github.com/mala-zaba/Adafruit_Python_DHT,
also see requirements.txt).
As this requires access to the GPIO, you will need to run home-assistant
as root.

Configuration:

To use the Adafruit DHT sensor you will need to
add something like the following to your config/configuration.yaml:

sensor:
  platform: dht
  sensor: DHT22
  pin: 23
  monitored_conditions:
    - temperature
    - humidity

Variables:

sensor
*Required
The sensor type, DHT11, DHT22 or AM2302

pin
*Required
The pin the sensor is connected to, something like
'P8_11' for Beaglebone, '23' for Raspberry Pi

monitored_conditions
*Optional
Conditions to monitor. Available conditions are temperature and humidity.
"""
import logging
from datetime import timedelta

from homeassistant.util import Throttle
from homeassistant.const import TEMP_FAHRENHEIT
from homeassistant.helpers.entity import Entity

# update this requirement to upstream as soon as it supports python3
REQUIREMENTS = ['http://github.com/mala-zaba/Adafruit_Python_DHT/archive/' +
                '4101340de8d2457dd194bca1e8d11cbfc237e919.zip']
_LOGGER = logging.getLogger(__name__)
SENSOR_TYPES = {
    'temperature': ['Temperature', ''],
    'humidity': ['Humidity', '%']
}
# Return cached results if last scan was less then this time ago
# DHT11 is able to deliver data once per second, DHT22 once every two
MIN_TIME_BETWEEN_UPDATES = timedelta(seconds=30)


def setup_platform(hass, config, add_devices, discovery_info=None):
    """ Get the DHT sensor. """

    try:
        import Adafruit_DHT

    except ImportError:
        _LOGGER.exception(
            "Unable to import Adafruit_DHT. "
            "Did you maybe not install the 'Adafruit_DHT' package?")

        return False

    SENSOR_TYPES['temperature'][1] = hass.config.temperature_unit
    unit = hass.config.temperature_unit
    available_sensors = {
        "DHT11": Adafruit_DHT.DHT11,
        "DHT22": Adafruit_DHT.DHT22,
        "AM2302": Adafruit_DHT.AM2302
    }
    sensor = available_sensors[config['sensor']]

    pin = config['pin']

    if not sensor or not pin:
        _LOGGER.error(
            "Config error "
            "Please check your settings for DHT, sensor not supported.")
        return None

    data = DHTClient(Adafruit_DHT, sensor, pin)
    dev = []
    try:
        for variable in config['monitored_conditions']:
            if variable not in SENSOR_TYPES:
                _LOGGER.error('Sensor type: "%s" does not exist', variable)
            else:
                dev.append(DHTSensor(data, variable, unit))
    except KeyError:
        pass

    add_devices(dev)


# pylint: disable=too-few-public-methods
class DHTSensor(Entity):

    """ Implements an DHT sensor. """

    def __init__(self, dht_client, sensor_type, temp_unit):
        self.client_name = 'DHT sensor'
        self._name = SENSOR_TYPES[sensor_type][0]
        self.dht_client = dht_client
        self.temp_unit = temp_unit
        self.type = sensor_type
        self._state = None
        self._unit_of_measurement = SENSOR_TYPES[sensor_type][1]
        self.update()

    @property
    def name(self):
        return '{} {}'.format(self.client_name, self._name)

    @property
    def state(self):
        """ Returns the state of the device. """
        return self._state

    @property
    def unit_of_measurement(self):
        """ Unit of measurement of this entity, if any. """
        return self._unit_of_measurement

    def update(self):
        """ Gets the latest data from the DHT and updates the states. """

        self.dht_client.update()
        data = self.dht_client.data

        if self.type == 'temperature':
            self._state = round(data['temperature'], 1)
            if self.temp_unit == TEMP_FAHRENHEIT:
                self._state = round(data['temperature'] * 1.8 + 32, 1)
        elif self.type == 'humidity':
            self._state = round(data['humidity'], 1)


class DHTClient(object):

    """ Gets the latest data from the DHT sensor. """

    def __init__(self, adafruit_dht, sensor, pin):
        self.adafruit_dht = adafruit_dht
        self.sensor = sensor
        self.pin = pin
        self.data = dict()

    @Throttle(MIN_TIME_BETWEEN_UPDATES)
    def update(self):
        """ Gets the latest data the DHT sensor. """
        humidity, temperature = self.adafruit_dht.read_retry(self.sensor,
                                                             self.pin)
        if temperature:
            self.data['temperature'] = temperature
        if humidity:
            self.data['humidity'] = humidity



# -*- coding: utf-8 -*-
"""
homeassistant.components.sensor.mqtt
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Allows to configure a MQTT sensor.

This generic sensor implementation uses the MQTT message payload
as the sensor value. If messages in this state_topic are published
with RETAIN flag, the sensor will receive an instant update with
last known value. Otherwise, the initial state will be undefined.

sensor:
  platform: mqtt
  name: "MQTT Sensor"
  state_topic: "home/bedroom/temperature"
  unit_of_measurement: "ºC"

Variables:

name
*Optional
The name of the sensor. Default is 'MQTT Sensor'.

state_topic
*Required
The MQTT topic subscribed to receive sensor values.

unit_of_measurement
*Optional
Defines the units of measurement of the sensor, if any.

"""

import logging
from homeassistant.helpers.entity import Entity
import homeassistant.components.mqtt as mqtt

_LOGGER = logging.getLogger(__name__)

DEFAULT_NAME = "MQTT Sensor"

DEPENDENCIES = ['mqtt']


# pylint: disable=unused-argument
def setup_platform(hass, config, add_devices_callback, discovery_info=None):
    """ Add MQTT Sensor """

    if config.get('state_topic') is None:
        _LOGGER.error("Missing required variable: state_topic")
        return False

    add_devices_callback([MqttSensor(
        hass,
        config.get('name', DEFAULT_NAME),
        config.get('state_topic'),
        config.get('unit_of_measurement'))])


class MqttSensor(Entity):
    """ Represents a sensor that can be updated using MQTT """
    def __init__(self, hass, name, state_topic, unit_of_measurement):
        self._state = "-"
        self._hass = hass
        self._name = name
        self._state_topic = state_topic
        self._unit_of_measurement = unit_of_measurement

        def message_received(topic, payload, qos):
            """ A new MQTT message has been received. """
            self._state = payload
            self.update_ha_state()

        mqtt.subscribe(hass, self._state_topic, message_received)

    @property
    def should_poll(self):
        """ No polling needed """
        return False

    @property
    def name(self):
        """ The name of the sensor """
        return self._name

    @property
    def unit_of_measurement(self):
        """ Unit this state is expressed in. """
        return self._unit_of_measurement

    @property
    def state(self):
        """ Returns the state of the entity. """
        return self._state



"""
homeassistant.components.frontend
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Provides a frontend for Home Assistant.
"""
import re
import os
import logging

from . import version
import homeassistant.util as util
from homeassistant.const import URL_ROOT, HTTP_OK

DOMAIN = 'frontend'
DEPENDENCIES = ['api']

INDEX_PATH = os.path.join(os.path.dirname(__file__), 'index.html.template')

_LOGGER = logging.getLogger(__name__)


FRONTEND_URLS = [
    URL_ROOT, '/logbook', '/history', '/devService', '/devState', '/devEvent']
STATES_URL = re.compile(r'/states(/([a-zA-Z\._\-0-9/]+)|)')


def setup(hass, config):
    """ Setup serving the frontend. """
    if 'http' not in hass.config.components:
        _LOGGER.error('Dependency http is not loaded')
        return False

    for url in FRONTEND_URLS:
        hass.http.register_path('GET', url, _handle_get_root, False)

    hass.http.register_path('GET', STATES_URL, _handle_get_root, False)

    # Static files
    hass.http.register_path(
        'GET', re.compile(r'/static/(?P<file>[a-zA-Z\._\-0-9/]+)'),
        _handle_get_static, False)
    hass.http.register_path(
        'HEAD', re.compile(r'/static/(?P<file>[a-zA-Z\._\-0-9/]+)'),
        _handle_get_static, False)

    return True


def _handle_get_root(handler, path_match, data):
    """ Renders the debug interface. """

    handler.send_response(HTTP_OK)
    handler.send_header('Content-type', 'text/html; charset=utf-8')
    handler.end_headers()

    if handler.server.development:
        app_url = "home-assistant-polymer/src/home-assistant.html"
    else:
        app_url = "frontend-{}.html".format(version.VERSION)

    # auto login if no password was set, else check api_password param
    auth = ('no_password_set' if handler.server.no_password_set
            else data.get('api_password', ''))

    with open(INDEX_PATH) as template_file:
        template_html = template_file.read()

    template_html = template_html.replace('{{ app_url }}', app_url)
    template_html = template_html.replace('{{ auth }}', auth)

    handler.wfile.write(template_html.encode("UTF-8"))


def _handle_get_static(handler, path_match, data):
    """ Returns a static file for the frontend. """
    req_file = util.sanitize_path(path_match.group('file'))

    # Strip md5 hash out of frontend filename
    if re.match(r'^frontend-[A-Za-z0-9]{32}\.html$', req_file):
        req_file = "frontend.html"

    path = os.path.join(os.path.dirname(__file__), 'www_static', req_file)

    handler.write_file(path)



"""
homeassistant.components.media_player.chromecast
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Provides functionality to interact with Cast devices on the network.

WARNING: This platform is currently not working due to a changed Cast API
"""
import logging

from homeassistant.const import (
    STATE_PLAYING, STATE_PAUSED, STATE_IDLE, STATE_OFF,
    STATE_UNKNOWN, CONF_HOST)

from homeassistant.components.media_player import (
    MediaPlayerDevice,
    SUPPORT_PAUSE, SUPPORT_VOLUME_SET, SUPPORT_VOLUME_MUTE,
    SUPPORT_TURN_ON, SUPPORT_TURN_OFF, SUPPORT_YOUTUBE,
    SUPPORT_PREVIOUS_TRACK, SUPPORT_NEXT_TRACK,
    MEDIA_TYPE_MUSIC, MEDIA_TYPE_TVSHOW, MEDIA_TYPE_VIDEO)

REQUIREMENTS = ['pychromecast==0.6.12']
CONF_IGNORE_CEC = 'ignore_cec'
CAST_SPLASH = 'https://home-assistant.io/images/cast/splash.png'
SUPPORT_CAST = SUPPORT_PAUSE | SUPPORT_VOLUME_SET | SUPPORT_VOLUME_MUTE | \
    SUPPORT_TURN_ON | SUPPORT_TURN_OFF | SUPPORT_PREVIOUS_TRACK | \
    SUPPORT_NEXT_TRACK | SUPPORT_YOUTUBE
KNOWN_HOSTS = []

# pylint: disable=invalid-name
cast = None


# pylint: disable=unused-argument
def setup_platform(hass, config, add_devices, discovery_info=None):
    """ Sets up the cast platform. """
    global cast
    import pychromecast
    cast = pychromecast

    logger = logging.getLogger(__name__)

    # import CEC IGNORE attributes
    ignore_cec = config.get(CONF_IGNORE_CEC, [])
    if isinstance(ignore_cec, list):
        cast.IGNORE_CEC += ignore_cec
    else:
        logger.error('Chromecast conig, %s must be a list.', CONF_IGNORE_CEC)

    hosts = []

    if discovery_info and discovery_info[0] not in KNOWN_HOSTS:
        hosts = [discovery_info[0]]

    elif CONF_HOST in config:
        hosts = [config[CONF_HOST]]

    else:
        hosts = (host_port[0] for host_port
                 in cast.discover_chromecasts()
                 if host_port[0] not in KNOWN_HOSTS)

    casts = []

    for host in hosts:
        try:
            casts.append(CastDevice(host))
        except cast.ChromecastConnectionError:
            pass
        else:
            KNOWN_HOSTS.append(host)

    add_devices(casts)


class CastDevice(MediaPlayerDevice):
    """ Represents a Cast device on the network. """

    # pylint: disable=too-many-public-methods

    def __init__(self, host):
        import pychromecast.controllers.youtube as youtube
        self.cast = cast.Chromecast(host)
        self.youtube = youtube.YouTubeController()
        self.cast.register_handler(self.youtube)

        self.cast.socket_client.receiver_controller.register_status_listener(
            self)
        self.cast.socket_client.media_controller.register_status_listener(self)

        self.cast_status = self.cast.status
        self.media_status = self.cast.media_controller.status

    # Entity properties and methods

    @property
    def should_poll(self):
        return False

    @property
    def name(self):
        """ Returns the name of the device. """
        return self.cast.device.friendly_name

    # MediaPlayerDevice properties and methods

    @property
    def state(self):
        """ State of the player. """
        if self.media_status is None:
            return STATE_UNKNOWN
        elif self.media_status.player_is_playing:
            return STATE_PLAYING
        elif self.media_status.player_is_paused:
            return STATE_PAUSED
        elif self.media_status.player_is_idle:
            return STATE_IDLE
        elif self.cast.is_idle:
            return STATE_OFF
        else:
            return STATE_UNKNOWN

    @property
    def volume_level(self):
        """ Volume level of the media player (0..1). """
        return self.cast_status.volume_level if self.cast_status else None

    @property
    def is_volume_muted(self):
        """ Boolean if volume is currently muted. """
        return self.cast_status.volume_muted if self.cast_status else None

    @property
    def media_content_id(self):
        """ Content ID of current playing media. """
        return self.media_status.content_id if self.media_status else None

    @property
    def media_content_type(self):
        """ Content type of current playing media. """
        if self.media_status is None:
            return None
        elif self.media_status.media_is_tvshow:
            return MEDIA_TYPE_TVSHOW
        elif self.media_status.media_is_movie:
            return MEDIA_TYPE_VIDEO
        elif self.media_status.media_is_musictrack:
            return MEDIA_TYPE_MUSIC
        return None

    @property
    def media_duration(self):
        """ Duration of current playing media in seconds. """
        return self.media_status.duration if self.media_status else None

    @property
    def media_image_url(self):
        """ Image url of current playing media. """
        if self.media_status is None:
            return None

        images = self.media_status.images

        return images[0].url if images else None

    @property
    def media_title(self):
        """ Title of current playing media. """
        return self.media_status.title if self.media_status else None

    @property
    def media_artist(self):
        """ Artist of current playing media. (Music track only) """
        return self.media_status.artist if self.media_status else None

    @property
    def media_album(self):
        """ Album of current playing media. (Music track only) """
        return self.media_status.album_name if self.media_status else None

    @property
    def media_album_artist(self):
        """ Album arist of current playing media. (Music track only) """
        return self.media_status.album_artist if self.media_status else None

    @property
    def media_track(self):
        """ Track number of current playing media. (Music track only) """
        return self.media_status.track if self.media_status else None

    @property
    def media_series_title(self):
        """ Series title of current playing media. (TV Show only)"""
        return self.media_status.series_title if self.media_status else None

    @property
    def media_season(self):
        """ Season of current playing media. (TV Show only) """
        return self.media_status.season if self.media_status else None

    @property
    def media_episode(self):
        """ Episode of current playing media. (TV Show only) """
        return self.media_status.episode if self.media_status else None

    @property
    def app_id(self):
        """  ID of the current running app. """
        return self.cast.app_id

    @property
    def app_name(self):
        """  Name of the current running app. """
        return self.cast.app_display_name

    @property
    def supported_media_commands(self):
        """ Flags of media commands that are supported. """
        return SUPPORT_CAST

    def turn_on(self):
        """ Turns on the ChromeCast. """
        # The only way we can turn the Chromecast is on is by launching an app
        if not self.cast.status or not self.cast.status.is_active_input:
            if self.cast.app_id:
                self.cast.quit_app()

            self.cast.play_media(
                CAST_SPLASH, cast.STREAM_TYPE_BUFFERED)

    def turn_off(self):
        """ Turns Chromecast off. """
        self.cast.quit_app()

    def mute_volume(self, mute):
        """ mute the volume. """
        self.cast.set_volume_muted(mute)

    def set_volume_level(self, volume):
        """ set volume level, range 0..1. """
        self.cast.set_volume(volume)

    def media_play(self):
        """ Send play commmand. """
        self.cast.media_controller.play()

    def media_pause(self):
        """ Send pause command. """
        self.cast.media_controller.pause()

    def media_previous_track(self):
        """ Send previous track command. """
        self.cast.media_controller.rewind()

    def media_next_track(self):
        """ Send next track command. """
        self.cast.media_controller.skip()

    def media_seek(self, position):
        """ Seek the media to a specific location. """
        self.cast.media_controller.seek(position)

    def play_youtube(self, media_id):
        """ Plays a YouTube media. """
        self.youtube.play_video(media_id)

    # implementation of chromecast status_listener methods

    def new_cast_status(self, status):
        """ Called when a new cast status is received. """
        self.cast_status = status
        self.update_ha_state()

    def new_media_status(self, status):
        """ Called when a new media status is received. """
        self.media_status = status
        self.update_ha_state()



"""
homeassistant.components.switch.rpi_gpio
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Allows to control the GPIO pins of a Raspberry Pi.
Note: To use RPi GPIO, Home Assistant must be run as root.

Configuration:

switch:
  platform: rpi_gpio
  invert_logic: false
  ports:
    11: Fan Office
    12: Light Desk

Variables:

invert_logic
*Optional
If true, inverts the output logic to ACTIVE LOW. Default is false (ACTIVE HIGH)

ports
*Required
An array specifying the GPIO ports to use and the name to use in the frontend.
"""

import logging
try:
    import RPi.GPIO as GPIO
except ImportError:
    GPIO = None
from homeassistant.helpers.entity import ToggleEntity
from homeassistant.const import (DEVICE_DEFAULT_NAME,
                                 EVENT_HOMEASSISTANT_START,
                                 EVENT_HOMEASSISTANT_STOP)

DEFAULT_INVERT_LOGIC = False

REQUIREMENTS = ['RPi.GPIO==0.5.11']
_LOGGER = logging.getLogger(__name__)


# pylint: disable=unused-argument
def setup_platform(hass, config, add_devices, discovery_info=None):
    """ Sets up the Raspberry PI GPIO ports. """
    if GPIO is None:
        _LOGGER.error('RPi.GPIO not available. rpi_gpio ports ignored.')
        return
    # pylint: disable=no-member
    GPIO.setmode(GPIO.BCM)

    switches = []
    invert_logic = config.get('invert_logic', DEFAULT_INVERT_LOGIC)
    ports = config.get('ports')
    for port_num, port_name in ports.items():
        switches.append(RPiGPIOSwitch(port_name, port_num, invert_logic))
    add_devices(switches)

    def cleanup_gpio(event):
        """ Stuff to do before stop home assistant. """
        # pylint: disable=no-member
        GPIO.cleanup()

    def prepare_gpio(event):
        """ Stuff to do when home assistant starts. """
        hass.bus.listen_once(EVENT_HOMEASSISTANT_STOP, cleanup_gpio)

    hass.bus.listen_once(EVENT_HOMEASSISTANT_START, prepare_gpio)


class RPiGPIOSwitch(ToggleEntity):
    """ Represents a port that can be toggled using Raspberry Pi GPIO. """

    def __init__(self, name, gpio, invert_logic):
        self._name = name or DEVICE_DEFAULT_NAME
        self._gpio = gpio
        self._active_state = not invert_logic
        self._state = not self._active_state
        # pylint: disable=no-member
        GPIO.setup(gpio, GPIO.OUT)

    @property
    def name(self):
        """ The name of the port. """
        return self._name

    @property
    def should_poll(self):
        """ No polling needed. """
        return False

    @property
    def is_on(self):
        """ True if device is on. """
        return self._state

    def turn_on(self, **kwargs):
        """ Turn the device on. """
        if self._switch(self._active_state):
            self._state = True
        self.update_ha_state()

    def turn_off(self, **kwargs):
        """ Turn the device off. """
        if self._switch(not self._active_state):
            self._state = False
        self.update_ha_state()

    def _switch(self, new_state):
        """ Change the output value to Raspberry Pi GPIO port. """
        _LOGGER.info('Setting GPIO %s to %s', self._gpio, new_state)
        # pylint: disable=bare-except
        try:
            # pylint: disable=no-member
            GPIO.output(self._gpio, 1 if new_state else 0)
        except:
            _LOGGER.error('GPIO "%s" output failed', self._gpio)
            return False
        return True

    # pylint: disable=no-self-use
    @property
    def device_state_attributes(self):
        """ Returns device specific state attributes. """
        return None

    @property
    def state_attributes(self):
        """ Returns optional state attributes. """
        data = {}
        device_attr = self.device_state_attributes
        if device_attr is not None:
            data.update(device_attr)
        return data



"""
homeassistant.components.sensor.time_date
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Date and Time service.

Configuration:

To use the Date and Time sensor you will need to add something like the
following to your config/configuration.yaml

sensor:
  platform: time_date
  display_options:
    - 'time'
    - 'date'
    - 'date_time'
    - 'time_date'
    - 'time_utc'
    - 'beat'

Variables:

display_options
*Required
The variable you wish to display. See the configuration example above for a
list of all available variables.
"""
import logging

import homeassistant.util.dt as dt_util
from homeassistant.helpers.entity import Entity

_LOGGER = logging.getLogger(__name__)
OPTION_TYPES = {
    'time': 'Time',
    'date': 'Date',
    'date_time': 'Date & Time',
    'time_date': 'Time & Date',
    'beat': 'Time (beat)',
    'time_utc': 'Time (UTC)',
}


def setup_platform(hass, config, add_devices, discovery_info=None):
    """ Get the Time and Date sensor. """

    if hass.config.time_zone is None:
        _LOGGER.error("Timezone is not set in Home Assistant config")
        return False

    dev = []
    for variable in config['display_options']:
        if variable not in OPTION_TYPES:
            _LOGGER.error('Option type: "%s" does not exist', variable)
        else:
            dev.append(TimeDateSensor(variable))

    add_devices(dev)


# pylint: disable=too-few-public-methods
class TimeDateSensor(Entity):
    """ Implements a Time and Date sensor. """

    def __init__(self, option_type):
        self._name = OPTION_TYPES[option_type]
        self.type = option_type
        self._state = None
        self.update()

    @property
    def name(self):
        """ Returns the name of the device. """
        return self._name

    @property
    def state(self):
        """ Returns the state of the device. """
        return self._state

    def update(self):
        """ Gets the latest data and updates the states. """

        time_date = dt_util.utcnow()
        time = dt_util.datetime_to_time_str(dt_util.as_local(time_date))
        time_utc = dt_util.datetime_to_time_str(time_date)
        date = dt_util.datetime_to_date_str(dt_util.as_local(time_date))

        # Calculate the beat (Swatch Internet Time) time without date.
        hours, minutes, seconds = time_date.strftime('%H:%M:%S').split(':')
        beat = ((int(seconds) + (int(minutes) * 60) + ((int(hours) + 1) *
                                                       3600)) / 86.4)

        if self.type == 'time':
            self._state = time
        elif self.type == 'date':
            self._state = date
        elif self.type == 'date_time':
            self._state = date + ', ' + time
        elif self.type == 'time_date':
            self._state = time + ', ' + date
        elif self.type == 'time_utc':
            self._state = time_utc
        elif self.type == 'beat':
            self._state = '{0:.2f}'.format(beat)



"""
homeassistant.components.light.vera
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Support for Vera lights. This component is useful if you wish for switches
connected to your Vera controller to appear as lights in Home Assistant.
All switches will be added as a light unless you exclude them in the config.

Configuration:

To use the Vera lights you will need to add something like the following to
your config/configuration.yaml.

light:
    platform: vera
    vera_controller_url: http://YOUR_VERA_IP:3480/
    device_data:
        12:
            name: My awesome switch
            exclude: true
        13:
            name: Another switch

Variables:

vera_controller_url
*Required
This is the base URL of your vera controller including the port number if not
running on 80. Example: http://192.168.1.21:3480/

device_data
*Optional
This contains an array additional device info for your Vera devices. It is not
required and if not specified all lights configured in your Vera controller
will be added with default values. You should use the id of your vera device
as the key for the device within device_data.

These are the variables for the device_data array:

name
*Optional
This parameter allows you to override the name of your Vera device in the HA
interface, if not specified the value configured for the device in your Vera
will be used.

exclude
*Optional
This parameter allows you to exclude the specified device from Home Assistant,
it should be set to "true" if you want this device excluded.

"""
import logging
from requests.exceptions import RequestException
from homeassistant.components.switch.vera import VeraSwitch
# pylint: disable=no-name-in-module, import-error
import homeassistant.external.vera.vera as veraApi

_LOGGER = logging.getLogger(__name__)


# pylint: disable=unused-argument
def setup_platform(hass, config, add_devices_callback, discovery_info=None):
    """ Find and return Vera lights. """

    base_url = config.get('vera_controller_url')
    if not base_url:
        _LOGGER.error(
            "The required parameter 'vera_controller_url'"
            " was not found in config"
        )
        return False

    device_data = config.get('device_data', {})

    controller = veraApi.VeraController(base_url)
    devices = []
    try:
        devices = controller.get_devices(['Switch', 'On/Off Switch'])
    except RequestException:
        # There was a network related error connecting to the vera controller
        _LOGGER.exception("Error communicating with Vera API")
        return False

    lights = []
    for device in devices:
        extra_data = device_data.get(device.deviceId, {})
        exclude = extra_data.get('exclude', False)

        if exclude is not True:
            lights.append(VeraSwitch(device, extra_data))

    add_devices_callback(lights)



"""
homeassistant.components.thermostat
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Provides functionality to interact with thermostats.
"""
import logging

from homeassistant.helpers.entity_component import EntityComponent

import homeassistant.util as util
from homeassistant.helpers.entity import Entity
from homeassistant.helpers.temperature import convert
from homeassistant.const import (
    ATTR_ENTITY_ID, ATTR_TEMPERATURE, STATE_ON, STATE_OFF, TEMP_CELCIUS)

DOMAIN = "thermostat"
DEPENDENCIES = []

ENTITY_ID_FORMAT = DOMAIN + ".{}"
SCAN_INTERVAL = 60

SERVICE_SET_AWAY_MODE = "set_away_mode"
SERVICE_SET_TEMPERATURE = "set_temperature"

ATTR_CURRENT_TEMPERATURE = "current_temperature"
ATTR_AWAY_MODE = "away_mode"
ATTR_MAX_TEMP = "max_temp"
ATTR_MIN_TEMP = "min_temp"

_LOGGER = logging.getLogger(__name__)


def set_away_mode(hass, away_mode, entity_id=None):
    """ Turn all or specified thermostat away mode on. """
    data = {
        ATTR_AWAY_MODE: away_mode
    }

    if entity_id:
        data[ATTR_ENTITY_ID] = entity_id

    hass.services.call(DOMAIN, SERVICE_SET_AWAY_MODE, data)


def set_temperature(hass, temperature, entity_id=None):
    """ Set new target temperature. """
    data = {ATTR_TEMPERATURE: temperature}

    if entity_id is not None:
        data[ATTR_ENTITY_ID] = entity_id

    hass.services.call(DOMAIN, SERVICE_SET_TEMPERATURE, data)


def setup(hass, config):
    """ Setup thermostats. """
    component = EntityComponent(_LOGGER, DOMAIN, hass, SCAN_INTERVAL)
    component.setup(config)

    def thermostat_service(service):
        """ Handles calls to the services. """

        # Convert the entity ids to valid light ids
        target_thermostats = component.extract_from_service(service)

        if service.service == SERVICE_SET_AWAY_MODE:
            away_mode = service.data.get(ATTR_AWAY_MODE)

            if away_mode is None:
                _LOGGER.error(
                    "Received call to %s without attribute %s",
                    SERVICE_SET_AWAY_MODE, ATTR_AWAY_MODE)

            elif away_mode:
                for thermostat in target_thermostats:
                    thermostat.turn_away_mode_on()
            else:
                for thermostat in target_thermostats:
                    thermostat.turn_away_mode_off()

        elif service.service == SERVICE_SET_TEMPERATURE:
            temperature = util.convert(
                service.data.get(ATTR_TEMPERATURE), float)

            if temperature is None:
                return

            for thermostat in target_thermostats:
                thermostat.set_temperature(convert(
                    temperature, hass.config.temperature_unit,
                    thermostat.unit_of_measurement))

        for thermostat in target_thermostats:
            thermostat.update_ha_state(True)

    hass.services.register(
        DOMAIN, SERVICE_SET_AWAY_MODE, thermostat_service)

    hass.services.register(
        DOMAIN, SERVICE_SET_TEMPERATURE, thermostat_service)

    return True


class ThermostatDevice(Entity):
    """ Represents a thermostat within Home Assistant. """

    # pylint: disable=no-self-use

    @property
    def state(self):
        """ Returns the current state. """
        return self.target_temperature

    @property
    def device_state_attributes(self):
        """ Returns device specific state attributes. """
        return None

    @property
    def state_attributes(self):
        """ Returns optional state attributes. """

        thermostat_unit = self.unit_of_measurement
        user_unit = self.hass.config.temperature_unit

        data = {
            ATTR_CURRENT_TEMPERATURE: round(convert(self.current_temperature,
                                                    thermostat_unit,
                                                    user_unit), 1),
            ATTR_MIN_TEMP: round(convert(self.min_temp,
                                         thermostat_unit,
                                         user_unit), 0),
            ATTR_MAX_TEMP: round(convert(self.max_temp,
                                         thermostat_unit,
                                         user_unit), 0)
        }

        is_away = self.is_away_mode_on

        if is_away is not None:
            data[ATTR_AWAY_MODE] = STATE_ON if is_away else STATE_OFF

        device_attr = self.device_state_attributes

        if device_attr is not None:
            data.update(device_attr)

        return data

    @property
    def unit_of_measurement(self):
        """ Unit of measurement this thermostat expresses itself in. """
        return NotImplementedError

    @property
    def current_temperature(self):
        """ Returns the current temperature. """
        raise NotImplementedError

    @property
    def target_temperature(self):
        """ Returns the temperature we try to reach. """
        raise NotImplementedError

    @property
    def is_away_mode_on(self):
        """
        Returns if away mode is on.
        Return None if no away mode available.
        """
        return None

    def set_temperate(self, temperature):
        """ Set new target temperature. """
        pass

    def turn_away_mode_on(self):
        """ Turns away mode on. """
        pass

    def turn_away_mode_off(self):
        """ Turns away mode off. """
        pass

    @property
    def min_temp(self):
        """ Return minimum temperature. """
        return convert(7, TEMP_CELCIUS, self.unit_of_measurement)

    @property
    def max_temp(self):
        """ Return maxmum temperature. """
        return convert(35, TEMP_CELCIUS, self.unit_of_measurement)



# -*- coding: utf-8 -*-
"""
homeassistant.components.switch.command_switch
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Allows to configure custom shell commands to turn a switch on/off.
"""
import logging
from homeassistant.components.switch import SwitchDevice
import subprocess

_LOGGER = logging.getLogger(__name__)


# pylint: disable=unused-argument
def setup_platform(hass, config, add_devices_callback, discovery_info=None):
    """ Find and return switches controlled by shell commands. """

    switches = config.get('switches', {})
    devices = []

    for dev_name, properties in switches.items():
        devices.append(
            CommandSwitch(
                dev_name,
                properties.get('oncmd', 'true'),
                properties.get('offcmd', 'true')))

    add_devices_callback(devices)


class CommandSwitch(SwitchDevice):
    """ Represents a switch that can be togggled using shell commands """
    def __init__(self, name, command_on, command_off):
        self._name = name
        self._state = False
        self._command_on = command_on
        self._command_off = command_off

    @staticmethod
    def _switch(command):
        """ Execute the actual commands """
        _LOGGER.info('Running command: %s', command)

        success = (subprocess.call(command, shell=True) == 0)

        if not success:
            _LOGGER.error('Command failed: %s', command)

        return success

    @property
    def should_poll(self):
        """ No polling needed """
        return False

    @property
    def name(self):
        """ The name of the switch """
        return self._name

    @property
    def is_on(self):
        """ True if device is on. """
        return self._state

    def turn_on(self, **kwargs):
        """ Turn the device on. """
        if CommandSwitch._switch(self._command_on):
            self._state = True
        self.update_ha_state()

    def turn_off(self, **kwargs):
        """ Turn the device off. """
        if CommandSwitch._switch(self._command_off):
            self._state = False
        self.update_ha_state()



__author__ = 'jamespcole'

import requests
import json
from requests.exceptions import RequestException

class SabnzbdApi(object):
	def __init__(self, base_url, api_key):
		if not base_url.endswith('/'):
			base_url = base_url + '/'
		self.BASE_URL = base_url	
		self.api_key = api_key
		self.queue = {}

	def refresh_queue(self):
		try:
			api_args = {}
			api_args['apikey'] = self.api_key
			api_args['mode'] = 'queue'
			api_args['start'] = '0'
			api_args['limit'] = '10'
			api_args['output'] = 'json'

			url = self.BASE_URL + 'api'
			response = requests.get(url, params=api_args)

			self.queue = response.json().get('queue')
		except RequestException:
			raise SabnzbdApiException("Failed to communicate with SABnzbd API.")

	def check_available(self):
		try:	
			api_args = {}
			api_args['apikey'] = self.api_key
			api_args['mode'] = 'qstatus'
			api_args['output'] = 'json'

			url = self.BASE_URL + 'api'
			response = requests.get(url, params=api_args, timeout=5)
			json_obj = response.json();
			if json_obj.get('status', True) == False:
				raise SabnzbdApiException(json_obj.get('error', 'Failed to communicate with SABnzbd API.'))
		except RequestException:
			raise SabnzbdApiException("Failed to communicate with SABnzbd API.")

		return True

class SabnzbdApiException(Exception):
    pass



"""
homeassistant.components.zwave
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Connects Home Assistant to a Z-Wave network.
"""
from pprint import pprint

from homeassistant import bootstrap
from homeassistant.const import (
    EVENT_HOMEASSISTANT_START, EVENT_HOMEASSISTANT_STOP,
    EVENT_PLATFORM_DISCOVERED, ATTR_SERVICE, ATTR_DISCOVERED)

DOMAIN = "zwave"
DEPENDENCIES = []
REQUIREMENTS = ['pydispatcher==2.0.5']

CONF_USB_STICK_PATH = "usb_path"
DEFAULT_CONF_USB_STICK_PATH = "/zwaveusbstick"
CONF_DEBUG = "debug"

DISCOVER_SENSORS = "zwave.sensors"

COMMAND_CLASS_SENSOR_BINARY = 48
COMMAND_CLASS_SENSOR_MULTILEVEL = 49
COMMAND_CLASS_BATTERY = 128

# list of tuple (DOMAIN, discovered service, supported command classes)
DISCOVERY_COMPONENTS = [
    ('sensor', DISCOVER_SENSORS,
     [COMMAND_CLASS_SENSOR_BINARY, COMMAND_CLASS_SENSOR_MULTILEVEL]),
]

ATTR_NODE_ID = "node_id"
ATTR_VALUE_ID = "value_id"

NETWORK = None


def _obj_to_dict(obj):
    """ Converts an obj into a hash for debug. """
    return {key: getattr(obj, key) for key
            in dir(obj)
            if key[0] != '_' and not hasattr(getattr(obj, key), '__call__')}


def nice_print_node(node):
    """ Prints a nice formatted node to the output (debug method) """
    node_dict = _obj_to_dict(node)
    node_dict['values'] = {value_id: _obj_to_dict(value)
                           for value_id, value in node.values.items()}

    print("\n\n\n")
    print("FOUND NODE", node.product_name)
    pprint(node_dict)
    print("\n\n\n")


def setup(hass, config):
    """
    Setup Z-wave.
    Will automatically load components to support devices found on the network.
    """
    # pylint: disable=global-statement, import-error
    global NETWORK

    from pydispatch import dispatcher
    from openzwave.option import ZWaveOption
    from openzwave.network import ZWaveNetwork

    use_debug = str(config[DOMAIN].get(CONF_DEBUG)) == '1'

    # Setup options
    options = ZWaveOption(
        config[DOMAIN].get(CONF_USB_STICK_PATH, DEFAULT_CONF_USB_STICK_PATH),
        user_path=hass.config.config_dir)

    options.set_console_output(use_debug)
    options.lock()

    NETWORK = ZWaveNetwork(options, autostart=False)

    if use_debug:
        def log_all(signal, value=None):
            """ Log all the signals. """
            print("")
            print("SIGNAL *****", signal)
            if value and signal in (ZWaveNetwork.SIGNAL_VALUE_CHANGED,
                                    ZWaveNetwork.SIGNAL_VALUE_ADDED):
                pprint(_obj_to_dict(value))
            print("")

        dispatcher.connect(log_all, weak=False)

    def value_added(node, value):
        """ Called when a value is added to a node on the network. """
        for component, discovery_service, command_ids in DISCOVERY_COMPONENTS:
            if value.command_class in command_ids:
                # Ensure component is loaded
                bootstrap.setup_component(hass, component, config)

                # Fire discovery event
                hass.bus.fire(EVENT_PLATFORM_DISCOVERED, {
                    ATTR_SERVICE: discovery_service,
                    ATTR_DISCOVERED: {
                        ATTR_NODE_ID: node.node_id,
                        ATTR_VALUE_ID: value.value_id,
                    }
                })

    dispatcher.connect(
        value_added, ZWaveNetwork.SIGNAL_VALUE_ADDED, weak=False)

    def stop_zwave(event):
        """ Stop Z-wave. """
        NETWORK.stop()

    def start_zwave(event):
        """ Called when Home Assistant starts up. """
        NETWORK.start()

        hass.bus.listen_once(EVENT_HOMEASSISTANT_STOP, stop_zwave)

    hass.bus.listen_once(EVENT_HOMEASSISTANT_START, start_zwave)

    return True



"""
homeassistant.components.sensor.vera
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Support for Vera sensors.

Configuration:

To use the Vera sensors you will need to add something like the following to
your config/configuration.yaml

sensor:
    platform: vera
    vera_controller_url: http://YOUR_VERA_IP:3480/
    device_data:
        12:
            name: My awesome sensor
            exclude: true
        13:
            name: Another sensor

Variables:

vera_controller_url
*Required
This is the base URL of your vera controller including the port number if not
running on 80
Example: http://192.168.1.21:3480/


device_data
*Optional
This contains an array additional device info for your Vera devices.  It is not
required and if not specified all sensors configured in your Vera controller
will be added with default values.  You should use the id of your vera device
as the key for the device within device_data

These are the variables for the device_data array:

name
*Optional
This parameter allows you to override the name of your Vera device in the HA
interface, if not specified the value configured for the device in your Vera
will be used


exclude
*Optional
This parameter allows you to exclude the specified device from homeassistant,
it should be set to "true" if you want this device excluded

"""
import logging
from requests.exceptions import RequestException
import homeassistant.util.dt as dt_util

from homeassistant.helpers.entity import Entity
from homeassistant.const import (
    ATTR_BATTERY_LEVEL, ATTR_TRIPPED, ATTR_ARMED, ATTR_LAST_TRIP_TIME,
    TEMP_CELCIUS, TEMP_FAHRENHEIT)
# pylint: disable=no-name-in-module, import-error
import homeassistant.external.vera.vera as veraApi

_LOGGER = logging.getLogger(__name__)


# pylint: disable=unused-argument
def get_devices(hass, config):
    """ Find and return Vera Sensors. """

    base_url = config.get('vera_controller_url')
    if not base_url:
        _LOGGER.error(
            "The required parameter 'vera_controller_url'"
            " was not found in config"
        )
        return False

    device_data = config.get('device_data', {})

    vera_controller = veraApi.VeraController(base_url)
    categories = ['Temperature Sensor', 'Light Sensor', 'Sensor']
    devices = []
    try:
        devices = vera_controller.get_devices(categories)
    except RequestException:
        # There was a network related error connecting to the vera controller
        _LOGGER.exception("Error communicating with Vera API")
        return False

    vera_sensors = []
    for device in devices:
        extra_data = device_data.get(device.deviceId, {})
        exclude = extra_data.get('exclude', False)

        if exclude is not True:
            vera_sensors.append(VeraSensor(device, extra_data))

    return vera_sensors


def setup_platform(hass, config, add_devices, discovery_info=None):
    """ Performs setup for Vera controller devices. """
    add_devices(get_devices(hass, config))


class VeraSensor(Entity):
    """ Represents a Vera Sensor. """

    def __init__(self, vera_device, extra_data=None):
        self.vera_device = vera_device
        self.extra_data = extra_data
        if self.extra_data and self.extra_data.get('name'):
            self._name = self.extra_data.get('name')
        else:
            self._name = self.vera_device.name
        self.current_value = ''
        self._temperature_units = None

    def __str__(self):
        return "%s %s %s" % (self.name, self.vera_device.deviceId, self.state)

    @property
    def state(self):
        return self.current_value

    @property
    def name(self):
        """ Get the mame of the sensor. """
        return self._name

    @property
    def unit_of_measurement(self):
        """ Unit of measurement of this entity, if any. """
        return self._temperature_units

    @property
    def state_attributes(self):
        attr = super().state_attributes
        if self.vera_device.has_battery:
            attr[ATTR_BATTERY_LEVEL] = self.vera_device.battery_level + '%'

        if self.vera_device.is_armable:
            armed = self.vera_device.refresh_value('Armed')
            attr[ATTR_ARMED] = 'True' if armed == '1' else 'False'

        if self.vera_device.is_trippable:
            last_tripped = self.vera_device.refresh_value('LastTrip')
            if last_tripped is not None:
                utc_time = dt_util.utc_from_timestamp(int(last_tripped))
                attr[ATTR_LAST_TRIP_TIME] = dt_util.datetime_to_str(
                    utc_time)
            else:
                attr[ATTR_LAST_TRIP_TIME] = None
            tripped = self.vera_device.refresh_value('Tripped')
            attr[ATTR_TRIPPED] = 'True' if tripped == '1' else 'False'

        attr['Vera Device Id'] = self.vera_device.vera_device_id
        return attr

    def update(self):
        if self.vera_device.category == "Temperature Sensor":
            self.vera_device.refresh_value('CurrentTemperature')
            current_temp = self.vera_device.get_value('CurrentTemperature')
            vera_temp_units = self.vera_device.veraController.temperature_units

            if vera_temp_units == 'F':
                self._temperature_units = TEMP_FAHRENHEIT
            else:
                self._temperature_units = TEMP_CELCIUS

            if self.hass:
                temp = self.hass.config.temperature(
                    current_temp,
                    self._temperature_units)

                current_temp, self._temperature_units = temp

            self.current_value = current_temp
        elif self.vera_device.category == "Light Sensor":
            self.vera_device.refresh_value('CurrentLevel')
            self.current_value = self.vera_device.get_value('CurrentLevel')
        elif self.vera_device.category == "Sensor":
            tripped = self.vera_device.refresh_value('Tripped')
            self.current_value = 'Tripped' if tripped == '1' else 'Not Tripped'
        else:
            self.current_value = 'Unknown'



"""
homeassistant.components.modbus
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Support for Modbus sensors.

Configuration:
To use the Modbus sensors you will need to add something like the following to
your config/configuration.yaml

sensor:
    platform: modbus
    slave: 1
    registers:
        16:
            name: My integer sensor
            unit: C
        24:
            bits:
                0:
                    name: My boolean sensor
                2:
                    name: My other boolean sensor
    coils:
        0:
            name: My coil switch

Variables:

slave
*Required
Slave number (ignored and can be omitted if not serial Modbus).

unit
*Required
Unit to attach to value (optional, ignored for boolean sensors).

registers
*Required
Contains a list of relevant registers to read from. It can contain a
"bits" section, listing relevant bits.

coils
*Optional
Contains a list of relevant coils to read from.

Note:
- Each named register will create an integer sensor.
- Each named bit will create a boolean sensor.
"""

import logging

import homeassistant.components.modbus as modbus
from homeassistant.helpers.entity import Entity
from homeassistant.const import (
    TEMP_CELCIUS, TEMP_FAHRENHEIT,
    STATE_ON, STATE_OFF)

_LOGGER = logging.getLogger(__name__)
DEPENDENCIES = ['modbus']


def setup_platform(hass, config, add_devices, discovery_info=None):
    """ Read config and create Modbus devices """
    sensors = []
    slave = config.get("slave", None)
    if modbus.TYPE == "serial" and not slave:
        _LOGGER.error("No slave number provided for serial Modbus")
        return False
    registers = config.get("registers")
    if registers:
        for regnum, register in registers.items():
            if register.get("name"):
                sensors.append(ModbusSensor(register.get("name"),
                                            slave,
                                            regnum,
                                            None,
                                            register.get("unit")))
            if register.get("bits"):
                bits = register.get("bits")
                for bitnum, bit in bits.items():
                    if bit.get("name"):
                        sensors.append(ModbusSensor(bit.get("name"),
                                                    slave,
                                                    regnum,
                                                    bitnum))
    coils = config.get("coils")
    if coils:
        for coilnum, coil in coils.items():
            sensors.append(ModbusSensor(coil.get("name"),
                                        slave,
                                        coilnum,
                                        coil=True))

    add_devices(sensors)


class ModbusSensor(Entity):
    # pylint: disable=too-many-arguments
    """ Represents a Modbus Sensor """

    def __init__(self, name, slave, register, bit=None, unit=None, coil=False):
        self._name = name
        self.slave = int(slave) if slave else 1
        self.register = int(register)
        self.bit = int(bit) if bit else None
        self._value = None
        self._unit = unit
        self._coil = coil

    def __str__(self):
        return "%s: %s" % (self.name, self.state)

    @property
    def should_poll(self):
        """ We should poll, because slaves are not allowed to
            initiate communication on Modbus networks"""
        return True

    @property
    def unique_id(self):
        """ Returns a unique id. """
        return "MODBUS-SENSOR-{}-{}-{}".format(self.slave,
                                               self.register,
                                               self.bit)

    @property
    def state(self):
        """ Returns the state of the sensor. """
        if self.bit:
            return STATE_ON if self._value else STATE_OFF
        else:
            return self._value

    @property
    def name(self):
        """ Get the name of the sensor. """
        return self._name

    @property
    def unit_of_measurement(self):
        """ Unit of measurement of this entity, if any. """
        if self._unit == "C":
            return TEMP_CELCIUS
        elif self._unit == "F":
            return TEMP_FAHRENHEIT
        else:
            return self._unit

    def update(self):
        """ Update the state of the sensor. """
        if self._coil:
            result = modbus.NETWORK.read_coils(self.register, 1)
            self._value = result.bits[0]
        else:
            result = modbus.NETWORK.read_holding_registers(
                unit=self.slave, address=self.register,
                count=1)
            val = 0
            for i, res in enumerate(result.registers):
                val += res * (2**(i*16))
            if self.bit:
                self._value = val & (0x0001 << self.bit)
            else:
                self._value = val
